@@ -18,36 +18,53 @@
 #include "util.hpp"
 #include "version.hpp"
 
+#include <osmium/util/string.hpp>
 #include <osmium/version.hpp>
 
 #include <CLI/CLI.hpp>
 
-#ifdef HAVE_LUA
 #include <lua.hpp>
-#endif
 
 #include <algorithm>
-#include <array>
+#include <cmath>
 #include <cstdio>
 #include <cstring>
 #include <stdexcept>
 #include <thread> // for number of threads
+#include <vector>
 
-static osmium::Box parse_bbox_param(std::string const &arg)
+namespace {
+
+void error_bbox()
 {
-    double minx = NAN;
-    double maxx = NAN;
-    double miny = NAN;
-    double maxy = NAN;
+    throw std::runtime_error{"Bounding box must be specified like: "
+                             "minlon,minlat,maxlon,maxlat."};
+}
 
-    int const n =
-        sscanf(arg.c_str(), "%lf,%lf,%lf,%lf", &minx, &miny, &maxx, &maxy);
+double parse_and_check_coordinate(std::string const &str)
+{
+    char *end = nullptr;
 
-    if (n != 4) {
-        throw std::runtime_error{"Bounding box must be specified like: "
-                                 "minlon,minlat,maxlon,maxlat."};
+    double const value = std::strtod(str.c_str(), &end);
+    if (end != &*str.end() || !std::isfinite(value)) {
+        error_bbox();
     }
 
+    return value;
+}
+
+osmium::Box parse_bbox_param(std::string const &arg)
+{
+    auto const values = osmium::split_string(arg, ',', true);
+    if (values.size() != 4) {
+        error_bbox();
+    }
+
+    double const minx = parse_and_check_coordinate(values[0]);
+    double const miny = parse_and_check_coordinate(values[1]);
+    double const maxx = parse_and_check_coordinate(values[2]);
+    double const maxy = parse_and_check_coordinate(values[3]);
+
     if (maxx <= minx) {
         throw std::runtime_error{
             "Bounding box failed due to maxlon <= minlon."};
@@ -60,12 +77,16 @@ static osmium::Box parse_bbox_param(std::string const &arg)
 
     log_debug("Applying bounding box: {},{} to {},{}", minx, miny, maxx, maxy);
 
-    return osmium::Box{minx, miny, maxx, maxy};
+    osmium::Box const box{minx, miny, maxx, maxy};
+    if (!box.valid()) {
+        error_bbox();
+    }
+
+    return box;
 }
 
-static void parse_expire_tiles_param(char const *arg,
-                                     uint32_t *expire_tiles_zoom_min,
-                                     uint32_t *expire_tiles_zoom)
+void parse_expire_tiles_param(char const *arg, uint32_t *expire_tiles_zoom_min,
+                              uint32_t *expire_tiles_zoom)
 {
     if (!arg || arg[0] == '-') {
         throw std::runtime_error{"Missing argument for option --expire-tiles."
@@ -112,33 +133,11 @@ static void parse_expire_tiles_param(char const *arg,
                              " tile expiry must be separated by '-'."};
 }
 
-void print_version()
+void check_options_non_slim(CLI::App const &app)
 {
-    fmt::print(stderr, "Build: {}\n", get_build_type());
-    fmt::print(stderr, "Compiled using the following library versions:\n");
-    fmt::print(stderr, "Libosmium {}\n", LIBOSMIUM_VERSION_STRING);
-    fmt::print(stderr, "Proj {}\n", get_proj_version());
-#ifdef HAVE_LUA
-#ifdef HAVE_LUAJIT
-    fmt::print(stderr, "{} ({})\n", LUA_RELEASE, LUAJIT_VERSION);
-#else
-    fmt::print(stderr, "{}\n", LUA_RELEASE);
-#endif
-#else
-    fmt::print(stderr, "Lua support not included\n");
-#endif
-}
-
-static void check_options_non_slim(CLI::App const &app)
-{
-    std::array<std::string, 7> const slim_options = {
-        "--flat-nodes",
-        "--middle-database-format",
-        "--middle-schema",
-        "--middle-with-nodes",
-        "--middle-way-node-index-id-shift",
-        "--tablespace-slim-data",
-        "--tablespace-slim-index"};
+    std::vector<std::string> const slim_options = {
+        "--cache", "--middle-schema", "--middle-with-nodes",
+        "--tablespace-slim-data", "--tablespace-slim-index"};
 
     for (auto const &opt : slim_options) {
         if (app.count(opt) > 0) {
@@ -148,7 +147,7 @@ static void check_options_non_slim(CLI::App const &app)
     }
 }
 
-static void check_options_output_flex(CLI::App const &app)
+void check_options_output_flex(CLI::App const &app)
 {
     auto const ignored_options = app.get_options([](CLI::Option const *option) {
         return option->get_group() == "Pgsql output options" ||
@@ -164,7 +163,7 @@ static void check_options_output_flex(CLI::App const &app)
     }
 }
 
-static void check_options_output_null(CLI::App const &app)
+void check_options_output_null(CLI::App const &app)
 {
     auto const ignored_options = app.get_options([](CLI::Option const *option) {
         return option->get_group() == "Pgsql output options" ||
@@ -182,7 +181,7 @@ static void check_options_output_null(CLI::App const &app)
     }
 }
 
-static void check_options_output_pgsql(CLI::App const &app, options_t *options)
+void check_options_output_pgsql(CLI::App const &app, options_t *options)
 {
     if (app.count("--latlong") + app.count("--merc") + app.count("--proj") >
         1) {
@@ -206,21 +205,12 @@ static void check_options_output_pgsql(CLI::App const &app, options_t *options)
     }
 }
 
-static void check_options(options_t *options)
+void check_options(options_t *options)
 {
-    if (options->append && options->create) {
-        throw std::runtime_error{"--append and --create options can not be "
-                                 "used at the same time!"};
-    }
-
     if (options->append && !options->slim) {
         throw std::runtime_error{"--append can only be used with slim mode!"};
     }
 
-    if (options->droptemp && !options->slim) {
-        throw std::runtime_error{"--drop only makes sense with --slim."};
-    }
-
     if (options->cache < 0) {
         options->cache = 0;
         log_warn("RAM cache cannot be negative. Using 0 instead.");
@@ -238,7 +228,7 @@ static void check_options(options_t *options)
     }
 }
 
-static void check_options_expire(options_t *options) {
+void check_options_expire(options_t *options) {
     // Zoom level 31 is the technical limit because we use 32-bit integers for
     // the x and y index of a tile ID.
     if (options->expire_tiles_zoom_min > 31) {
@@ -261,6 +251,22 @@ static void check_options_expire(options_t *options) {
     }
 }
 
+} // anonymous namespace
+
+void print_version()
+{
+    fmt::print(stderr, "osm2pgsql version {}\n", get_osm2pgsql_version());
+    fmt::print(stderr, "Build: {}\n", get_build_type());
+    fmt::print(stderr, "Compiled using the following library versions:\n");
+    fmt::print(stderr, "Libosmium {}\n", LIBOSMIUM_VERSION_STRING);
+    fmt::print(stderr, "Proj {}\n", get_proj_version());
+#ifdef HAVE_LUAJIT
+    fmt::print(stderr, "{} ({})\n", LUA_RELEASE, LUAJIT_VERSION);
+#else
+    fmt::print(stderr, "{}\n", LUA_RELEASE);
+#endif
+}
+
 // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,hicpp-avoid-c-arrays,modernize-avoid-c-arrays)
 options_t parse_command_line(int argc, char *argv[])
 {
@@ -291,7 +297,7 @@ options_t parse_command_line(int argc, char *argv[])
         ->description("Update existing osm2pgsql database (needs --slim).");
 
     // --create
-    app.add_flag("-c,--create", options.create)
+    app.add_flag("-c,--create")
         ->description("Import OSM data from file into database. This is the "
                       "default if --append is not used.");
 
@@ -338,8 +344,7 @@ options_t parse_command_line(int argc, char *argv[])
 
     // --output
     app.add_option("-O,--output", options.output_backend)
-        ->description("Set output ('pgsql' (default), 'flex', 'gazetteer' "
-                      "(deprecated), 'null').")
+        ->description("Set output ('pgsql' (default), 'flex', 'null').")
         ->type_name("OUTPUT")
         ->group("Output options");
 
@@ -456,7 +461,6 @@ options_t parse_command_line(int argc, char *argv[])
         ->description("Compute area column using Web Mercator coordinates.")
         ->group("Pgsql output options");
 
-#ifdef HAVE_LUA
     // --tag-transform-script
     app.add_option("--tag-transform-script", options.tag_transform_script)
         ->description(
@@ -464,7 +468,6 @@ options_t parse_command_line(int argc, char *argv[])
         ->option_text("SCRIPT")
         ->check(CLI::ExistingFile)
         ->group("Pgsql output options");
-#endif
 
     // ----------------------------------------------------------------------
     // Expire options
@@ -508,7 +511,7 @@ options_t parse_command_line(int argc, char *argv[])
 
     // --drop
     app.add_flag("--drop", options.droptemp)
-        ->description("Drop middle tables after import (needs --slim).")
+        ->description("Drop middle tables and flat node file after import.")
         ->group("Middle options");
 
     // --extra-attributes
@@ -519,8 +522,8 @@ options_t parse_command_line(int argc, char *argv[])
 
     // --flat-nodes
     app.add_option("-F,--flat-nodes", options.flat_node_file)
-        ->description("File for storing node locations (needs --slim,"
-                      " default: store in database).")
+        ->description(
+            "File for storing node locations (default: store in database).")
         ->type_name("FILE")
         ->group("Middle options");
 
@@ -531,13 +534,6 @@ options_t parse_command_line(int argc, char *argv[])
         ->type_name("SCHEMA")
         ->group("Middle options");
 
-    std::string middle_database_format_raw;
-    // --middle-database-format
-    app.add_option("--middle-database-format", middle_database_format_raw)
-        ->description("Set middle db format ('legacy', 'new' (default)).")
-        ->type_name("FORMAT")
-        ->group("Middle options");
-
     // --middle-with-nodes
     app.add_flag("--middle-with-nodes", options.middle_with_nodes)
         ->description("Store tagged nodes in db (new middle db format only).")
@@ -575,13 +571,6 @@ options_t parse_command_line(int argc, char *argv[])
         ->description("Disable concurrent index creation.")
         ->group("Advanced options");
 
-    // --middle-way-node-index-id-shift
-    app.add_option("--middle-way-node-index-id-shift",
-                   options.way_node_index_id_shift)
-        ->description("Set ID shift for bucket index.")
-        ->type_name("N")
-        ->group("Advanced options");
-
     // --number-processes
     app.add_option("--number-processes", options.num_procs)
         // The threads will open up database connections which will
@@ -598,13 +587,6 @@ options_t parse_command_line(int argc, char *argv[])
     // Tablespace options
     // ----------------------------------------------------------------------
 
-    std::string tablespace_index;
-    app.add_option("-i,--tablespace-index", tablespace_index)
-        ->description("Tablespace for indexes (sets default for "
-                      "--tablespace-(main|slim)-index).")
-        ->option_text("TBLSPC")
-        ->group("Tablespace options");
-
     app.add_option("--tablespace-main-data", options.tblsmain_data)
         ->description("Tablespace for main tables.")
         ->option_text("TBLSPC")
@@ -625,44 +607,6 @@ options_t parse_command_line(int argc, char *argv[])
         ->option_text("TBLSPC")
         ->group("Tablespace options");
 
-    // ----------------------------------------------------------------------
-    // Deprecated options
-    // ----------------------------------------------------------------------
-
-    // --cache-strategy
-    app.add_option_function<std::string>(
-           "--cache-strategy",
-           [&](std::string const &) {
-               log_warn("Deprecated option --cache-strategy ignored");
-           })
-        ->description(
-            "The option has been removed and only a warning is printed.")
-        ->option_text("TYPE")
-        ->group("Deprecated options");
-
-    // --with-forward-dependencies
-    app.add_option_function<std::string>(
-           "--with-forward-dependencies",
-           [&](std::string const &arg) {
-               log_warn("The option --with-forward-dependencies is deprecated "
-                        "and will soon be removed.");
-               if (arg == "false") {
-                   options.with_forward_dependencies = false;
-                   return;
-               }
-               if (arg == "true") {
-                   options.with_forward_dependencies = true;
-                   return;
-               }
-               throw fmt_error(
-                   "Unknown value for --with-forward-dependencies option: {}",
-                   arg);
-           })
-        ->description("Propagate changes from nodes to ways and node/way "
-                      "members to relations (default: true).")
-        ->option_text("BOOL")
-        ->group("Deprecated options");
-
     try {
         app.parse(argc, argv);
     } catch (...) {
@@ -676,51 +620,32 @@ options_t parse_command_line(int argc, char *argv[])
         return options;
     }
 
+    if (app.want_version()) {
+        options.command = command_t::version;
+        return options;
+    }
+
     log_info("osm2pgsql version {}", get_osm2pgsql_version());
 
     if (verbose) {
         get_logger().set_level(log_level::debug);
     }
 
-    if (app.want_version()) {
-        options.command = command_t::version;
-        return options;
+    if (options.append && app.count("--create")) {
+        throw std::runtime_error{"--append and --create options can not be "
+                                 "used at the same time!"};
     }
 
     check_options(&options);
 
     if (options.slim) { // slim mode, use database middle
-        if (middle_database_format_raw.empty()) {
-            // Database format not set on the command line, use defaults. For
-            // append mode we keep old default for backwards compatibility, for
-            // create mode the new format is the default. (These will possibly
-            // be overwritten later by properties setting if it exists.)
-            options.middle_database_format = options.append ? 1 : 2;
-        } else if (options.append) {
-            // Database format set on command line in append mode.
-            throw std::runtime_error{
-                "Do not use --middle-database-format with --append."};
-        } else {
-            // Database format set on the command line in create mode.
-            if (middle_database_format_raw == "legacy") {
-                options.middle_database_format = 1;
-            } else if (middle_database_format_raw == "new") {
-                options.middle_database_format = 2;
-            } else {
-                throw std::runtime_error{
-                    "Unknown value for --middle-database-format"
-                    " (Use 'legacy' or 'new')."};
-            }
-        }
+        options.middle_database_format = 2;
     } else { // non-slim mode, use ram middle
         check_options_non_slim(app);
     }
 
     if (options.output_backend == "flex") {
         check_options_output_flex(app);
-    } else if (options.output_backend == "gazetteer") {
-        log_warn(
-            "The 'gazetteer' output is deprecated and will soon be removed.");
     } else if (options.output_backend == "null") {
         check_options_output_null(app);
     } else if (options.output_backend == "pgsql" ||
@@ -732,18 +657,6 @@ options_t parse_command_line(int argc, char *argv[])
         options.input_format.clear();
     }
 
-    if (!tablespace_index.empty()) {
-        log_warn(
-            "The option -i, --tablespace-index is deprecated. Use "
-            "--tablespace-slim-index and/or --tablespace-main-index instead.");
-        if (options.tblsmain_index.empty()) {
-            options.tblsmain_index = tablespace_index;
-        }
-        if (options.tblsslim_index.empty()) {
-            options.tblsslim_index = tablespace_index;
-        }
-    }
-
     if (options.dbschema.empty()) {
         throw std::runtime_error{"Schema can not be empty."};
     }
