@@ -10,8 +10,10 @@
  * For a full list of authors see the git log.
  */
 
-#include "expire-output.hpp"
-#include "flex-table.hpp"
+#include <vector>
+
+class expire_output_t;
+class flex_table_t;
 
 void write_expire_output_list_to_debug_log(
     std::vector<expire_output_t> const &expire_outputs);
