@@ -13,6 +13,7 @@
 #include "tile.hpp"
 
 #include <cassert>
+#include <cstddef>
 #include <cstdint>
 #include <string>
 #include <utility>
@@ -76,7 +77,7 @@ public:
     /**
      * Create table for tiles.
      */
-    void create_output_table(pg_conn_t const &connection) const;
+    void create_output_table(pg_conn_t const &db_connection) const;
 
 private:
     /// The filename (if any) for output
