@@ -8,6 +8,7 @@
  */
 
 #include "flex-lua-geom.hpp"
+#include "geom-box.hpp"
 #include "geom-functions.hpp"
 #include "geom-pole-of-inaccessibility.hpp"
 #include "lua-utils.hpp"
@@ -50,10 +51,12 @@ int geom_gc(lua_State *lua_state) noexcept
     return 0;
 }
 
+namespace {
+
 // The following functions are called when their counterparts in Lua are
 // called on geometry objects.
 
-static int geom_area(lua_State *lua_state)
+int geom_area(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -66,7 +69,7 @@ static int geom_area(lua_State *lua_state)
     return 1;
 }
 
-static int geom_spherical_area(lua_State *lua_state)
+int geom_spherical_area(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -84,7 +87,7 @@ static int geom_spherical_area(lua_State *lua_state)
     return 1;
 }
 
-static int geom_length(lua_State *lua_state)
+int geom_length(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     try {
@@ -96,7 +99,7 @@ static int geom_length(lua_State *lua_state)
     return 1;
 }
 
-static int geom_centroid(lua_State *lua_state)
+int geom_centroid(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -110,7 +113,7 @@ static int geom_centroid(lua_State *lua_state)
     return 1;
 }
 
-static int geom_geometry_n(lua_State *lua_state)
+int geom_geometry_n(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     auto const index = static_cast<int>(luaL_checkinteger(lua_state, 2));
@@ -125,7 +128,7 @@ static int geom_geometry_n(lua_State *lua_state)
     return 1;
 }
 
-static int geom_geometry_type(lua_State *lua_state)
+int geom_geometry_type(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -139,14 +142,14 @@ static int geom_geometry_type(lua_State *lua_state)
     return 1;
 }
 
-static int geom_is_null(lua_State *lua_state)
+int geom_is_null(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     lua_pushboolean(lua_state, input_geometry->is_null());
     return 1;
 }
 
-static int geom_reverse(lua_State *lua_state)
+int geom_reverse(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -160,7 +163,7 @@ static int geom_reverse(lua_State *lua_state)
     return 1;
 }
 
-static int geom_line_merge(lua_State *lua_state)
+int geom_line_merge(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -174,7 +177,7 @@ static int geom_line_merge(lua_State *lua_state)
     return 1;
 }
 
-static int geom_num_geometries(lua_State *lua_state)
+int geom_num_geometries(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     lua_pushinteger(lua_state,
@@ -182,7 +185,7 @@ static int geom_num_geometries(lua_State *lua_state)
     return 1;
 }
 
-static int geom_pole_of_inaccessibility(lua_State *lua_state)
+int geom_pole_of_inaccessibility(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
 
@@ -210,7 +213,7 @@ static int geom_pole_of_inaccessibility(lua_State *lua_state)
     return 1;
 }
 
-static int geom_segmentize(lua_State *lua_state)
+int geom_segmentize(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     double const max_segment_length = luaL_checknumber(lua_state, 2);
@@ -225,7 +228,7 @@ static int geom_segmentize(lua_State *lua_state)
     return 1;
 }
 
-static int geom_simplify(lua_State *lua_state)
+int geom_simplify(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     double const tolerance = luaL_checknumber(lua_state, 2);
@@ -240,7 +243,24 @@ static int geom_simplify(lua_State *lua_state)
     return 1;
 }
 
-static int geom_srid(lua_State *lua_state)
+int geom_get_bbox(lua_State *lua_state)
+{
+    auto const *const input_geometry = unpack_geometry(lua_state);
+
+    try {
+        auto const box = geom::envelope(*input_geometry);
+        lua_pushnumber(lua_state, box.min_x());
+        lua_pushnumber(lua_state, box.min_y());
+        lua_pushnumber(lua_state, box.max_x());
+        lua_pushnumber(lua_state, box.max_y());
+    } catch (...) {
+        return luaL_error(lua_state, "Unknown error in 'get_bbox()'.\n");
+    }
+
+    return 4;
+}
+
+int geom_srid(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     lua_pushinteger(lua_state,
@@ -251,12 +271,12 @@ static int geom_srid(lua_State *lua_state)
 // XXX Implementation for Lua __tostring function on geometries. Currently
 // just returns the type as string. This could be improved, for instance by
 // showing a WKT representation of the geometry.
-static int geom_tostring(lua_State *lua_state)
+int geom_tostring(lua_State *lua_state)
 {
     return geom_geometry_type(lua_state);
 }
 
-static int geom_transform(lua_State *lua_state)
+int geom_transform(lua_State *lua_state)
 {
     auto const *const input_geometry = unpack_geometry(lua_state);
     auto const srid = static_cast<int>(luaL_checkinteger(lua_state, 2));
@@ -275,6 +295,8 @@ static int geom_transform(lua_State *lua_state)
     return 1;
 }
 
+} // anonymous namespace
+
 void init_geometry_class(lua_State *lua_state)
 {
     lua_getglobal(lua_state, "osm2pgsql");
@@ -294,6 +316,7 @@ void init_geometry_class(lua_State *lua_state)
     luaX_add_table_func(lua_state, "area", geom_area);
     luaX_add_table_func(lua_state, "length", geom_length);
     luaX_add_table_func(lua_state, "centroid", geom_centroid);
+    luaX_add_table_func(lua_state, "get_bbox", geom_get_bbox);
     luaX_add_table_func(lua_state, "geometry_n", geom_geometry_n);
     luaX_add_table_func(lua_state, "geometry_type", geom_geometry_type);
     luaX_add_table_func(lua_state, "is_null", geom_is_null);
