@@ -8,29 +8,35 @@
  */
 
 #include "flex-lua-index.hpp"
+
+#include "flex-index.hpp"
+#include "flex-table.hpp"
+#include "format.hpp"
 #include "lua-utils.hpp"
-#include "output-flex.hpp"
 #include "pgsql-capabilities.hpp"
 #include "util.hpp"
 
+#include <stdexcept>
 #include <string>
 #include <vector>
 
-static void check_and_add_column(flex_table_t const &table,
-                                 std::vector<std::string> *columns,
-                                 char const *column_name)
+namespace {
+
+void check_and_add_column(flex_table_t const &table,
+                          std::vector<std::string> *columns,
+                          char const *column_name)
 {
-    auto const *column = util::find_by_name(table, column_name);
+    auto const *column = util::find_by_name(table.columns(), column_name);
     if (!column) {
         throw fmt_error("Unknown column '{}' in table '{}'.", column_name,
                         table.name());
     }
-    columns->push_back(column_name);
+    columns->emplace_back(column_name);
 }
 
-static void check_and_add_columns(flex_table_t const &table,
-                                  std::vector<std::string> *columns,
-                                  lua_State *lua_state)
+void check_and_add_columns(flex_table_t const &table,
+                           std::vector<std::string> *columns,
+                           lua_State *lua_state)
 {
     if (!luaX_is_array(lua_state)) {
         throw std::runtime_error{
@@ -46,6 +52,8 @@ static void check_and_add_columns(flex_table_t const &table,
     });
 }
 
+} // anonymous namespace
+
 void flex_lua_setup_index(lua_State *lua_state, flex_table_t *table)
 {
     // get method
@@ -78,6 +86,12 @@ void flex_lua_setup_index(lua_State *lua_state, flex_table_t *table)
     }
     lua_pop(lua_state, 1);
 
+    // get name
+    std::string const name =
+        luaX_get_table_string(lua_state, "name", -1, "Index definition", "");
+    lua_pop(lua_state, 1);
+    index.set_name(name);
+
     // get expression
     std::string const expression = luaX_get_table_string(
         lua_state, "expression", -1, "Index definition", "");
