@@ -8,26 +8,29 @@
  */
 
 #include "flex-table-column.hpp"
+
 #include "format.hpp"
 #include "pgsql-capabilities.hpp"
 #include "util.hpp"
 
-#include <algorithm>
-#include <array>
+#include <cassert>
 #include <cctype>
 #include <cstdlib>
 #include <stdexcept>
 #include <utility>
+#include <vector>
+
+namespace {
 
 struct column_type_lookup
 {
     char const *m_name;
-    table_column_type type;
+    table_column_type m_type;
 
     char const *name() const noexcept { return m_name; }
 };
 
-static std::array<column_type_lookup, 26> const column_types = {
+std::vector<column_type_lookup> const column_types = {
     {{"text", table_column_type::text},
      {"boolean", table_column_type::boolean},
      {"bool", table_column_type::boolean},
@@ -51,27 +54,20 @@ static std::array<column_type_lookup, 26> const column_types = {
      {"multilinestring", table_column_type::multilinestring},
      {"multipolygon", table_column_type::multipolygon},
      {"geometrycollection", table_column_type::geometrycollection},
-     {"area", table_column_type::area},
      {"id_type", table_column_type::id_type},
      {"id_num", table_column_type::id_num}}};
 
-static table_column_type get_column_type_from_string(std::string const &type)
+table_column_type get_column_type_from_string(std::string const &type)
 {
     auto const *column_type = util::find_by_name(column_types, type);
     if (!column_type) {
         throw fmt_error("Unknown column type '{}'.", type);
     }
 
-    if (column_type->type == table_column_type::area) {
-        log_warn("The 'area' column type is deprecated. Please read");
-        log_warn("https://osm2pgsql.org/doc/tutorials/"
-                 "switching-from-add-row-to-insert/");
-    }
-
-    return column_type->type;
+    return column_type->m_type;
 }
 
-static std::string lowercase(std::string const &str)
+std::string lowercase(std::string const &str)
 {
     std::string result;
 
@@ -83,6 +79,8 @@ static std::string lowercase(std::string const &str)
     return result;
 }
 
+} // anonymous namespace
+
 flex_table_column_t::flex_table_column_t(std::string name,
                                          std::string const &type,
                                          std::string sql_type)
@@ -167,8 +165,6 @@ std::string flex_table_column_t::sql_type_name() const
         return fmt::format("Geometry(MULTIPOLYGON, {})", m_srid);
     case table_column_type::geometrycollection:
         return fmt::format("Geometry(GEOMETRYCOLLECTION, {})", m_srid);
-    case table_column_type::area:
-        return "real";
     case table_column_type::id_type:
         return "char(1)";
     case table_column_type::id_num:
