@@ -226,7 +226,8 @@ std::string flex_table_t::build_sql_column_list() const
 
 std::string flex_table_t::build_sql_create_id_index() const
 {
-    return fmt::format("CREATE INDEX ON {} USING BTREE ({}) {}", full_name(),
+    return fmt::format("CREATE {}INDEX ON {} USING BTREE ({}) {}",
+                       m_build_unique_id_index ? "UNIQUE " : "", full_name(),
                        id_column_names(),
                        tablespace_clause(index_tablespace()));
 }
@@ -242,12 +243,26 @@ bool flex_table_t::has_columns_with_expire() const noexcept
                        [](auto const &column) { return column.has_expire(); });
 }
 
-static void enable_check_trigger(pg_conn_t const &db_connection,
-                                 flex_table_t const &table)
+void flex_table_t::prepare(pg_conn_t const &db_connection) const
+{
+    if (has_id_column() && has_columns_with_expire()) {
+        db_connection.exec(build_sql_prepare_get_wkb());
+    }
+}
+
+void flex_table_t::analyze(pg_conn_t const &db_connection) const
+{
+    analyze_table(db_connection, schema(), name());
+}
+
+namespace {
+
+void enable_check_trigger(pg_conn_t const &db_connection,
+                          flex_table_t const &table)
 {
     std::string checks;
 
-    for (auto const &column : table) {
+    for (auto const &column : table.columns()) {
         if (column.is_geometry_column() && column.needs_isvalid()) {
             checks.append(fmt::format(
                 R"((NEW."{0}" IS NULL OR ST_IsValid(NEW."{0}")) AND )",
@@ -266,15 +281,18 @@ static void enable_check_trigger(pg_conn_t const &db_connection,
                               checks);
 }
 
-void table_connection_t::start(pg_conn_t const &db_connection, bool append)
+} // anonymous namespace
+
+void table_connection_t::start(pg_conn_t const &db_connection,
+                               bool append) const
 {
     if (!append) {
-        db_connection.exec("DROP TABLE IF EXISTS {} CASCADE",
-                           table().full_name());
+        drop_table_if_exists(db_connection, table().schema(), table().name());
     }
 
     // These _tmp tables can be left behind if we run out of disk space.
-    db_connection.exec("DROP TABLE IF EXISTS {}", table().full_tmp_name());
+    drop_table_if_exists(db_connection, table().schema(),
+                         table().name() + "_tmp");
 
     if (!append) {
         db_connection.exec(table().build_sql_create_table(
@@ -285,7 +303,7 @@ void table_connection_t::start(pg_conn_t const &db_connection, bool append)
         enable_check_trigger(db_connection, table());
     }
 
-    prepare(db_connection);
+    table().prepare(db_connection);
 }
 
 void table_connection_t::stop(pg_conn_t const &db_connection, bool updateable,
@@ -309,34 +327,14 @@ void table_connection_t::stop(pg_conn_t const &db_connection, bool updateable,
             flex_table_t::table_type::permanent, table().full_tmp_name()));
 
         std::string const columns = table().build_sql_column_list();
-        std::string sql = fmt::format("INSERT INTO {} ({}) SELECT {} FROM {}",
-                                      table().full_tmp_name(), columns, columns,
-                                      table().full_name());
-
-        auto const postgis_version = get_postgis_version();
 
         auto const geom_column_name =
             "\"" + table().geom_column().name() + "\"";
 
-        sql += " ORDER BY ";
-        if (postgis_version.major == 2 && postgis_version.minor < 4) {
-            log_debug("Using GeoHash for clustering table '{}'",
-                      table().name());
-            if (table().geom_column().srid() == 4326) {
-                sql += fmt::format("ST_GeoHash({},10)", geom_column_name);
-            } else {
-                sql += fmt::format(
-                    "ST_GeoHash(ST_Transform(ST_Envelope({}),4326),10)",
-                    geom_column_name);
-            }
-            sql += " COLLATE \"C\"";
-        } else {
-            log_debug("Using native order for clustering table '{}'",
-                      table().name());
-            // Since Postgis 2.4 the order function for geometries gives
-            // useful results.
-            sql += geom_column_name;
-        }
+        std::string const sql =
+            fmt::format("INSERT INTO {} ({}) SELECT {} FROM {} ORDER BY {}",
+                        table().full_tmp_name(), columns, columns,
+                        table().full_name(), geom_column_name);
 
         db_connection.exec(sql);
 
@@ -368,19 +366,7 @@ void table_connection_t::stop(pg_conn_t const &db_connection, bool updateable,
     }
 
     log_info("Analyzing table '{}'...", table().name());
-    analyze(db_connection);
-}
-
-void table_connection_t::prepare(pg_conn_t const &db_connection)
-{
-    if (table().has_id_column() && table().has_columns_with_expire()) {
-        db_connection.exec(table().build_sql_prepare_get_wkb());
-    }
-}
-
-void table_connection_t::analyze(pg_conn_t const &db_connection)
-{
-    analyze_table(db_connection, table().schema(), table().name());
+    table().analyze(db_connection);
 }
 
 void table_connection_t::create_id_index(pg_conn_t const &db_connection)
