@@ -20,7 +20,11 @@
 
 #include <osmium/osm/item_type.hpp>
 
+#include <cassert>
+#include <chrono>
 #include <cstddef>
+#include <cstdint>
+#include <future>
 #include <limits>
 #include <memory>
 #include <string>
@@ -32,7 +36,7 @@
  * output. This is not a real primary key, because the values are not
  * necessarily unique.
  */
-enum class flex_table_index_type {
+enum class flex_table_index_type : uint8_t {
     no_index,
     node, // index by node id
     way, // index by way id
@@ -54,7 +58,7 @@ public:
      * Table creation type: interim tables are created as UNLOGGED and with
      * autovacuum disabled.
      */
-    enum class table_type {
+    enum class table_type : uint8_t {
         interim,
         permanent
     };
@@ -111,14 +115,9 @@ public:
 
     std::size_t num_columns() const noexcept { return m_columns.size(); }
 
-    std::vector<flex_table_column_t>::const_iterator begin() const noexcept
+    std::vector<flex_table_column_t> const &columns() const noexcept
     {
-        return m_columns.begin();
-    }
-
-    std::vector<flex_table_column_t>::const_iterator end() const noexcept
-    {
-        return m_columns.end();
+        return m_columns;
     }
 
     flex_table_column_t *find_column_by_name(std::string const &name)
@@ -195,10 +194,24 @@ public:
         return m_always_build_id_index;
     }
 
+    void set_build_unique_id_index() noexcept
+    {
+        m_build_unique_id_index = true;
+    }
+
+    bool build_unique_id_index() const noexcept
+    {
+        return m_build_unique_id_index;
+    }
+
     bool has_columns_with_expire() const noexcept;
 
     std::size_t num() const noexcept { return m_table_num; }
 
+    void prepare(pg_conn_t const &db_connection) const;
+
+    void analyze(pg_conn_t const &db_connection) const;
+
 private:
     /// The schema this table is in
     std::string m_schema;
@@ -249,6 +262,9 @@ private:
     /// Always build the id index, not only when it is needed for updates?
     bool m_always_build_id_index = false;
 
+    /// Build the index as a unique index.
+    bool m_build_unique_id_index = false;
+
 }; // class flex_table_t
 
 class table_connection_t
@@ -264,16 +280,12 @@ public:
     {
     }
 
-    void start(pg_conn_t const &db_connection, bool append);
+    void start(pg_conn_t const &db_connection, bool append) const;
 
     void stop(pg_conn_t const &db_connection, bool updateable, bool append);
 
     flex_table_t const &table() const noexcept { return *m_table; }
 
-    void prepare(pg_conn_t const &db_connection);
-
-    void analyze(pg_conn_t const &db_connection);
-
     void create_id_index(pg_conn_t const &db_connection);
 
     /**
