@@ -14,8 +14,10 @@
 
 #include <fmt/args.h>
 
-gen_base_t::gen_base_t(pg_conn_t *connection, params_t *params)
-: m_connection(connection), m_params(params)
+#include <cassert>
+
+gen_base_t::gen_base_t(pg_conn_t *connection, bool append, params_t *params)
+: m_connection(connection), m_params(params), m_append(append)
 {
     assert(connection);
     assert(params);
@@ -82,7 +84,9 @@ std::string gen_base_t::context()
     return gen_name.empty() ? "" : fmt::format(" '{}'", gen_name);
 }
 
-static pg_result_t dbexec_internal(
+namespace {
+
+pg_result_t dbexec_internal(
     pg_conn_t const &connection, std::string const &templ,
     fmt::dynamic_format_arg_store<fmt::format_context> const &format_store)
 {
@@ -95,6 +99,8 @@ static pg_result_t dbexec_internal(
     }
 }
 
+} // anonymous namespace
+
 pg_result_t gen_base_t::dbexec(std::string const &templ)
 {
     fmt::dynamic_format_arg_store<fmt::format_context> format_store;
