@@ -10,17 +10,16 @@
  * For a full list of authors see the git log.
  */
 
-#include "gen-base.hpp"
-
 #include <memory>
 #include <string>
 
+class gen_base_t;
 class params_t;
 class pg_conn_t;
 
 /// Instantiate a generalizer for the specified strategy.
 std::unique_ptr<gen_base_t> create_generalizer(std::string const &strategy,
                                                pg_conn_t *connection,
-                                               params_t *params);
+                                               bool append, params_t *params);
 
 #endif // OSM2PGSQL_GEN_CREATE_HPP
