@@ -9,16 +9,18 @@
 
 #include "gen-discrete-isolation.hpp"
 
-#include "logging.hpp"
 #include "params.hpp"
 #include "pgsql.hpp"
 #include "util.hpp"
 
 #include <algorithm>
+#include <cmath>
+#include <cstdint>
+#include <cstdlib>
 #include <vector>
 
-gen_di_t::gen_di_t(pg_conn_t *connection, params_t *params)
-: gen_base_t(connection, params), m_timer_get(add_timer("get")),
+gen_di_t::gen_di_t(pg_conn_t *connection, bool append, params_t *params)
+: gen_base_t(connection, append, params), m_timer_get(add_timer("get")),
   m_timer_sort(add_timer("sort")), m_timer_di(add_timer("di")),
   m_timer_reorder(add_timer("reorder")), m_timer_write(add_timer("write"))
 {
@@ -115,7 +117,7 @@ FROM {src} WHERE {importance_column} > 0
                     min = dist;
                 }
             }
-            data[n].di = sqrt(min);
+            data[n].di = std::sqrt(min);
         }
         data[0].di = data[1].di + 1;
     }
@@ -143,7 +145,9 @@ FROM {src} WHERE {importance_column} > 0
     connection().exec("COMMIT");
     timer(m_timer_write).stop();
 
-    dbexec("ANALYZE {src}");
+    if (!append_mode()) {
+        dbexec("ANALYZE {src}");
+    }
 
     log_gen("Done.");
 }
