@@ -12,12 +12,16 @@
 
 #include "gen-base.hpp"
 
+#include <cstddef>
 #include <string_view>
 
+class params_t;
+class pg_conn_t;
+
 class gen_di_t : public gen_base_t
 {
 public:
-    gen_di_t(pg_conn_t *connection, params_t *params);
+    gen_di_t(pg_conn_t *connection, bool append, params_t *params);
 
     void process() override;
 
