@@ -19,7 +19,7 @@
 class gen_tile_builtup_t final : public gen_tile_t
 {
 public:
-    gen_tile_builtup_t(pg_conn_t *connection, params_t *params);
+    gen_tile_builtup_t(pg_conn_t *connection, bool append, params_t *params);
 
     ~gen_tile_builtup_t() override = default;
 
