@@ -15,7 +15,6 @@
  * in the future.
  */
 
-#include "canvas.hpp"
 #include "command-line-app.hpp"
 #include "debug-output.hpp"
 #include "expire-output.hpp"
@@ -30,7 +29,6 @@
 #include "lua-init.hpp"
 #include "lua-setup.hpp"
 #include "lua-utils.hpp"
-#include "options.hpp"
 #include "params.hpp"
 #include "pgsql-capabilities.hpp"
 #include "pgsql.hpp"
@@ -45,19 +43,16 @@
 #include <lua.hpp>
 
 #include <algorithm>
-#include <array>
-#include <cassert>
+#include <atomic>
 #include <chrono>
 #include <cstdint>
 #include <cstdlib>
-#include <cstring>
-#include <getopt.h>
-#include <limits>
+#include <memory>
+#include <mutex>
 #include <stdexcept>
 #include <string>
 #include <thread>
-
-constexpr std::size_t const max_force_single_thread = 4;
+#include <vector>
 
 // Lua can't call functions on C++ objects directly. This macro defines simple
 // C "trampoline" functions which are called from Lua which get the current
@@ -65,7 +60,7 @@ constexpr std::size_t const max_force_single_thread = 4;
 // context object.
 // NOLINTNEXTLINE(cppcoreguidelines-macro-usage)
 #define TRAMPOLINE(func_name, lua_name)                                        \
-    static int lua_trampoline_##func_name(lua_State *lua_state)                \
+    int lua_trampoline_##func_name(lua_State *lua_state)                       \
     {                                                                          \
         try {                                                                  \
             return static_cast<genproc_t *>(luaX_get_context(lua_state))       \
@@ -79,6 +74,10 @@ constexpr std::size_t const max_force_single_thread = 4;
         }                                                                      \
     }
 
+namespace {
+
+constexpr std::size_t const max_force_single_thread = 4;
+
 struct tile_extent
 {
     uint32_t xmin = 0;
@@ -88,27 +87,38 @@ struct tile_extent
     bool valid = false;
 };
 
-static bool table_is_empty(pg_conn_t const &db_connection,
-                           std::string const &schema, std::string const &table)
+bool table_is_empty(pg_conn_t const &db_connection, std::string const &schema,
+                    std::string const &table)
 {
     auto const result = db_connection.exec("SELECT 1 FROM {} LIMIT 1",
                                            qualified_name(schema, table));
     return result.num_tuples() == 0;
 }
 
-static tile_extent get_extent_from_db(pg_conn_t const &db_connection,
-                                      std::string const &schema,
-                                      std::string const &table,
-                                      std::string const &column, uint32_t zoom)
+tile_extent get_extent_from_db(pg_conn_t const &db_connection,
+                               std::string const &schema,
+                               std::string const &table,
+                               std::string const &column, bool raster,
+                               uint32_t zoom)
 {
     if (table_is_empty(db_connection, schema, table)) {
         return {};
     }
 
-    auto const result = db_connection.exec(
-        "SELECT ST_XMin(e), ST_YMin(e), ST_XMax(e), ST_YMax(e)"
-        " FROM ST_EstimatedExtent('{}', '{}', '{}') AS e",
-        schema, table, column);
+    pg_result_t result;
+    if (raster) {
+        result = db_connection.exec(
+            "SELECT ST_XMin(extent), ST_YMin(extent),"
+            " ST_XMax(extent), ST_YMax(extent)"
+            " FROM raster_columns WHERE r_table_schema='{}'"
+            " AND r_table_name='{}' AND r_raster_column = '{}'",
+            schema, table, column);
+    } else {
+        result = db_connection.exec(
+            "SELECT ST_XMin(e), ST_YMin(e), ST_XMax(e), ST_YMax(e)"
+            " FROM ST_EstimatedExtent('{}', '{}', '{}') AS e",
+            schema, table, column);
+    }
 
     if (result.num_tuples() == 0 || result.is_null(0, 0)) {
         return {};
@@ -127,11 +137,12 @@ static tile_extent get_extent_from_db(pg_conn_t const &db_connection,
             osmium::geom::mercy_to_tiley(zoom, extent_ymin), true};
 }
 
-static tile_extent get_extent_from_db(pg_conn_t const &db_connection,
-                                      std::string const &default_schema,
-                                      params_t const &params, uint32_t zoom)
+tile_extent get_extent_from_db(pg_conn_t const &db_connection,
+                               std::string const &default_schema,
+                               params_t const &params, uint32_t zoom)
 {
     auto const schema = params.get_string("schema", default_schema);
+
     std::string table;
     if (params.has("src_table")) {
         table = params.get_string("src_table");
@@ -144,14 +155,20 @@ static tile_extent get_extent_from_db(pg_conn_t const &db_connection,
     } else {
         throw std::runtime_error{"Need 'src_table' or 'src_tables' param."};
     }
+
     auto const geom_column = params.get_string("geom_column", "geom");
-    return get_extent_from_db(db_connection, schema, table, geom_column, zoom);
+    auto const raster_column = params.get_string("raster_column", "");
+
+    bool const is_raster = !raster_column.empty();
+
+    return get_extent_from_db(db_connection, schema, table,
+                              is_raster ? raster_column : geom_column,
+                              is_raster, zoom);
 }
 
-static void
-get_tiles_from_table(pg_conn_t const &connection, std::string const &table,
-                     uint32_t zoom,
-                     std::vector<std::pair<uint32_t, uint32_t>> *tiles)
+void get_tiles_from_table(pg_conn_t const &connection, std::string const &table,
+                          uint32_t zoom,
+                          std::vector<std::pair<uint32_t, uint32_t>> *tiles)
 {
     auto const result = connection.exec(
         R"(SELECT x, y FROM "{}" WHERE zoom = {})", table, zoom);
@@ -195,38 +212,49 @@ private:
     std::size_t m_num_tiles;
 };
 
-void run_tile_gen(connection_params_t const &connection_params,
+void run_tile_gen(std::atomic_flag *error_flag,
+                  connection_params_t const &connection_params,
                   gen_base_t *master_generalizer, params_t params,
                   uint32_t zoom,
                   std::vector<std::pair<uint32_t, uint32_t>> *queue,
                   std::mutex *mut, unsigned int n)
 {
-    logger::init_thread(n);
+    try {
+        logger::init_thread(n);
 
-    log_debug("Started generalizer thread for '{}'.",
-              master_generalizer->strategy());
-    pg_conn_t db_connection{connection_params, "gen.tile"};
-    std::string const strategy{master_generalizer->strategy()};
-    auto generalizer = create_generalizer(strategy, &db_connection, &params);
+        log_debug("Started generalizer thread for '{}'.",
+                  master_generalizer->strategy());
+        pg_conn_t db_connection{connection_params, "gen.tile"};
+        std::string const strategy{master_generalizer->strategy()};
+        auto generalizer =
+            create_generalizer(strategy, &db_connection,
+                               master_generalizer->append_mode(), &params);
 
-    while (true) {
-        std::pair<uint32_t, uint32_t> p;
-        {
-            std::lock_guard<std::mutex> const guard{*mut};
-            if (queue->empty()) {
-                master_generalizer->merge_timers(*generalizer);
-                break;
+        while (true) {
+            std::pair<uint32_t, uint32_t> p;
+            {
+                std::lock_guard<std::mutex> const guard{*mut};
+                if (queue->empty()) {
+                    master_generalizer->merge_timers(*generalizer);
+                    break;
+                }
+                p = queue->back();
+                queue->pop_back();
             }
-            p = queue->back();
-            queue->pop_back();
-        }
 
-        tile_t const tile{zoom, p.first, p.second};
-        log_debug("Processing tile {}/{}/{}...", tile.zoom(), tile.x(),
-                  tile.y());
-        generalizer->process(tile);
+            tile_t const tile{zoom, p.first, p.second};
+            log_debug("Processing tile {}/{}/{}...", tile.zoom(), tile.x(),
+                      tile.y());
+            generalizer->process(tile);
+        }
+        log_debug("Shutting down generalizer thread.");
+    } catch (std::exception const &e) {
+        log_error("{}", e.what());
+        error_flag->test_and_set();
+    } catch (...) {
+        log_error("Unknown exception in generalizer thread.");
+        error_flag->test_and_set();
     }
-    log_debug("Shutting down generalizer thread.");
 }
 
 class genproc_t
@@ -238,14 +266,6 @@ public:
 
     int app_define_table()
     {
-#if 0
-        if (m_calling_context != calling_context::main) {
-            throw std::runtime_error{
-                "Database tables have to be defined in the"
-                " main Lua code, not in any of the callbacks."};
-        }
-#endif
-
         return setup_flex_table(m_lua_state.get(), &m_tables, &m_expire_outputs,
                                 m_dbschema, true, m_append);
     }
@@ -279,10 +299,6 @@ public:
             params.set("schema", m_dbschema);
         }
 
-        if (m_append) {
-            params.set("delete_existing", true);
-        }
-
         write_to_debug_log(params, "Params (config):");
 
         log_debug("Connecting to database...");
@@ -290,7 +306,7 @@ public:
 
         log_debug("Creating generalizer...");
         auto generalizer =
-            create_generalizer(strategy, &db_connection, &params);
+            create_generalizer(strategy, &db_connection, m_append, &params);
 
         log_info("Running generalizer '{}' ({})...", generalizer->name(),
                  generalizer->strategy());
@@ -311,10 +327,9 @@ public:
 
         log_debug("Timers:");
         for (auto const &timer : generalizer->timers()) {
-            log_debug(fmt::format(
-                "  {:10} {:>10L}", timer.name() + ":",
-                std::chrono::duration_cast<std::chrono::milliseconds>(
-                    timer.elapsed())));
+            log_debug("  {:10} {:>10L}", timer.name() + ":",
+                      std::chrono::duration_cast<std::chrono::milliseconds>(
+                          timer.elapsed()));
         }
         log_info("Finished generalizer '{}' in {}.", generalizer->name(),
                  util::human_readable_duration(timer_gen.stop()));
@@ -493,16 +508,21 @@ private:
             log_debug("Running in multi-threaded mode.");
             std::mutex mut;
             std::vector<std::thread> threads;
+            std::atomic_flag error_flag = ATOMIC_FLAG_INIT;
             for (unsigned int n = 1;
                  n <= std::min(m_jobs, static_cast<uint32_t>(tile_list.size()));
                  ++n) {
-                threads.emplace_back(run_tile_gen, m_connection_params,
-                                     generalizer, params, zoom, &tile_list,
-                                     &mut, n);
+                threads.emplace_back(run_tile_gen, &error_flag,
+                                     m_connection_params, generalizer, params,
+                                     zoom, &tile_list, &mut, n);
             }
             for (auto &t : threads) {
                 t.join();
             }
+            if (error_flag.test_and_set()) {
+                throw std::runtime_error{
+                    "Error in generalizer thread. Stopping."};
+            }
         }
     }
 
@@ -608,6 +628,8 @@ void genproc_t::run()
     }
 }
 
+} // anonymous namespace
+
 // NOLINTNEXTLINE(bugprone-exception-escape)
 int main(int argc, char *argv[])
 {
@@ -707,7 +729,11 @@ int main(int argc, char *argv[])
         }
 
         properties_t properties{connection_params, middle_dbschema};
-        properties.load();
+        if (!properties.load()) {
+            throw std::runtime_error{
+                "Did not find table 'osm2pgsql_properties' in database. "
+                "Database too old? Wrong schema?"};
+        }
 
         if (style.empty()) {
             style = properties.get_string("style", "");
