@@ -252,24 +252,22 @@ namespace {
 class without_first
 {
 public:
-    explicit without_first(point_list_t const &list) : m_list(list) {}
+    explicit without_first(point_list_t const &list) : m_list(&list) {}
 
     point_list_t::const_iterator begin()
     {
-        assert(m_list.begin() != m_list.end());
-        return std::next(m_list.begin());
+        assert(m_list->begin() != m_list->end());
+        return std::next(m_list->begin());
     }
 
-    point_list_t::const_iterator end() { return m_list.end(); }
+    point_list_t::const_iterator end() { return m_list->end(); }
 
 private:
-    point_list_t const &m_list;
+    point_list_t const *m_list;
 }; // class without_first
 
-} // anonymous namespace
-
-static void split_linestring(linestring_t const &line, double split_at,
-                             multilinestring_t *output)
+void split_linestring(linestring_t const &line, double split_at,
+                      multilinestring_t *output)
 {
     double dist = 0;
     point_t prev_pt{line.front()};
@@ -320,6 +318,8 @@ static void split_linestring(linestring_t const &line, double split_at,
     }
 }
 
+} // anonymous namespace
+
 void segmentize(geometry_t *output, geometry_t const &input,
                 double max_segment_length)
 {
@@ -364,7 +364,9 @@ double area(geometry_t const &geom)
 
 /****************************************************************************/
 
-static double spherical_area(polygon_t const &geom)
+namespace {
+
+double spherical_area(polygon_t const &geom)
 {
     boost::geometry::strategy::area::spherical<> const spherical_earth{
         6371008.8};
@@ -378,6 +380,8 @@ static double spherical_area(polygon_t const &geom)
     return boost::geometry::area(sph_geom, spherical_earth);
 }
 
+} // anonymous namespace
+
 double spherical_area(geometry_t const &geom)
 {
     assert(geom.srid() == 4326);
@@ -433,6 +437,8 @@ public:
     void operator()(T) const
     {}
 
+    // false positive https://github.com/llvm/llvm-project/issues/74738
+    // NOLINTNEXTLINE(cppcoreguidelines-rvalue-reference-param-not-moved)
     void operator()(geom::collection_t &&geom) const
     {
         for (auto &&sgeom : geom) {
@@ -441,6 +447,8 @@ public:
     }
 
     template <typename T>
+    // false positive https://github.com/llvm/llvm-project/issues/74738
+    // NOLINTNEXTLINE(cppcoreguidelines-rvalue-reference-param-not-moved)
     void operator()(geom::multigeometry_t<T> &&geom) const
     {
         for (auto &&sgeom : geom) {
@@ -471,23 +479,25 @@ std::vector<geometry_t> split_multi(geometry_t &&geom, bool split_multi)
 
 /****************************************************************************/
 
-static void reverse(geom::nullgeom_t * /*output*/,
-                    geom::nullgeom_t const & /*input*/) noexcept
+namespace {
+
+void reverse(geom::nullgeom_t * /*output*/,
+             geom::nullgeom_t const & /*input*/) noexcept
 {}
 
-static void reverse(geom::point_t *output, geom::point_t const &input) noexcept
+void reverse(geom::point_t *output, geom::point_t const &input) noexcept
 {
     *output = input;
 }
 
-static void reverse(point_list_t *output, point_list_t const &input)
+void reverse(point_list_t *output, point_list_t const &input)
 {
     output->reserve(input.size());
     std::reverse_copy(input.cbegin(), input.cend(),
                       std::back_inserter(*output));
 }
 
-static void reverse(geom::polygon_t *output, geom::polygon_t const &input)
+void reverse(geom::polygon_t *output, geom::polygon_t const &input)
 {
     reverse(&output->outer(), input.outer());
     for (auto const &g : input.inners()) {
@@ -505,6 +515,8 @@ void reverse(geom::multigeometry_t<T> *output,
     }
 }
 
+} // anonymous namespace
+
 void reverse(geometry_t *output, geometry_t const &input)
 {
     output->set_srid(input.srid());
@@ -525,13 +537,15 @@ geometry_t reverse(geometry_t const &input)
 
 /****************************************************************************/
 
+namespace {
+
 /**
  * Add points specified by iterators to the linestring. If linestring is not
  * empty, do not add the first point returned by *it.
  */
 template <typename ITERATOR>
-static void add_nodes_to_linestring(linestring_t *linestring, ITERATOR it,
-                                    ITERATOR end)
+void add_nodes_to_linestring(linestring_t *linestring, ITERATOR it,
+                             ITERATOR end)
 {
     if (!linestring->empty()) {
         assert(it != end);
@@ -544,6 +558,8 @@ static void add_nodes_to_linestring(linestring_t *linestring, ITERATOR it,
     }
 }
 
+} // anonymous namespace
+
 void line_merge(geometry_t *output, geometry_t const &input)
 {
     if (input.is_linestring()) {
@@ -593,10 +609,10 @@ void line_merge(geometry_t *output, geometry_t const &input)
     struct connection_t
     {
         std::size_t left = NOCONN;
-        linestring_t const *ls;
+        linestring_t const *linestring;
         std::size_t right = NOCONN;
 
-        explicit connection_t(linestring_t const *l) noexcept : ls(l) {}
+        explicit connection_t(linestring_t const *l) noexcept : linestring(l) {}
     };
 
     std::vector<connection_t> conns;
@@ -633,7 +649,7 @@ void line_merge(geometry_t *output, geometry_t const &input)
     std::size_t done_ways = 0;
     std::size_t const todo_ways = conns.size();
     for (std::size_t i = 0; i < todo_ways; ++i) {
-        if (!conns[i].ls ||
+        if (!conns[i].linestring ||
             (conns[i].left != NOCONN && conns[i].right != NOCONN)) {
             continue; // way already done or not the beginning of a segment
         }
@@ -645,8 +661,8 @@ void line_merge(geometry_t *output, geometry_t const &input)
 
             do {
                 auto &conn = conns[cur];
-                assert(conn.ls);
-                auto const &nl = *conn.ls;
+                assert(conn.linestring);
+                auto const &nl = *conn.linestring;
                 bool const forward = conn.left == prev;
                 prev = cur;
                 // add line
@@ -660,7 +676,7 @@ void line_merge(geometry_t *output, geometry_t const &input)
                     cur = conn.left;
                 }
                 // mark line as done
-                conns[prev].ls = nullptr;
+                conns[prev].linestring = nullptr;
                 ++done_ways;
             } while (cur != NOCONN);
         }
@@ -675,7 +691,7 @@ void line_merge(geometry_t *output, geometry_t const &input)
         // oh dear, there must be circular ways without an end
         // need to do the same shebang again
         for (std::size_t i = 0; i < todo_ways; ++i) {
-            if (!conns[i].ls) {
+            if (!conns[i].linestring) {
                 continue; // way already done
             }
 
@@ -686,12 +702,12 @@ void line_merge(geometry_t *output, geometry_t const &input)
 
                 do {
                     auto &conn = conns[cur];
-                    assert(conn.ls);
-                    auto const &nl = *conn.ls;
+                    assert(conn.linestring);
+                    auto const &nl = *conn.linestring;
                     bool const forward =
                         (conn.left == prev &&
-                         (!conns[conn.left].ls ||
-                          conns[conn.left].ls->back() == nl.front()));
+                         (!conns[conn.left].linestring ||
+                          conns[conn.left].linestring->back() == nl.front()));
                     prev = cur;
                     if (forward) {
                         // add line forwards
@@ -705,7 +721,7 @@ void line_merge(geometry_t *output, geometry_t const &input)
                         cur = conn.left;
                     }
                     // mark line as done
-                    conns[prev].ls = nullptr;
+                    conns[prev].linestring = nullptr;
                 } while (cur != i);
             }
 
@@ -728,6 +744,8 @@ geometry_t line_merge(geometry_t const &input)
 
 /****************************************************************************/
 
+namespace {
+
 /**
  * This helper function is used to calculate centroids of geometry collections.
  * It first creates a multi geometry that only contains the geometries of
@@ -740,7 +758,7 @@ geometry_t line_merge(geometry_t const &input)
  * Nested geometry collections are not allowed.
  */
 template <std::size_t N, typename T>
-static void filtered_centroid(collection_t const &collection, point_t *center)
+void filtered_centroid(collection_t const &collection, point_t *center)
 {
     multigeometry_t<T> multi;
     for (auto const &geom : collection) {
@@ -758,6 +776,8 @@ static void filtered_centroid(collection_t const &collection, point_t *center)
     boost::geometry::centroid(multi, *center);
 }
 
+} // anonymous namespace
+
 geometry_t centroid(geometry_t const &geom)
 {
     geom::geometry_t output{point_t{}, geom.srid()};
@@ -785,8 +805,9 @@ geometry_t centroid(geometry_t const &geom)
 
 /****************************************************************************/
 
-static bool simplify(linestring_t *output, linestring_t const &input,
-                     double tolerance)
+namespace {
+
+bool simplify(linestring_t *output, linestring_t const &input, double tolerance)
 {
     boost::geometry::simplify(input, *output, tolerance);
 
@@ -798,12 +819,12 @@ static bool simplify(linestring_t *output, linestring_t const &input,
     return output->size() > 1;
 }
 
-static bool simplify(multilinestring_t *output, multilinestring_t const &input,
-                     double tolerance)
+bool simplify(multilinestring_t *output, multilinestring_t const &input,
+              double tolerance)
 {
-    for (auto const &ls : input) {
+    for (auto const &linestring : input) {
         linestring_t simplified_ls;
-        if (simplify(&simplified_ls, ls, tolerance)) {
+        if (simplify(&simplified_ls, linestring, tolerance)) {
             output->add_geometry(std::move(simplified_ls));
         }
     }
@@ -811,11 +832,13 @@ static bool simplify(multilinestring_t *output, multilinestring_t const &input,
 }
 
 template <typename T>
-static bool simplify(T * /*output*/, T const & /*input*/, double /*tolerance*/)
+bool simplify(T * /*output*/, T const & /*input*/, double /*tolerance*/)
 {
     return false;
 }
 
+} // anonymous namespace
+
 void simplify(geometry_t *output, geometry_t const &input, double tolerance)
 {
     output->set_srid(input.srid());
