@@ -18,10 +18,11 @@
 #include <fmt/color.h>
 
 #include <atomic>
+#include <cstdint>
 #include <cstdio>
 #include <utility>
 
-enum class log_level
+enum class log_level : uint8_t
 {
     debug = 1,
     info = 2,
@@ -36,9 +37,10 @@ enum class log_level
 class logger
 {
 public:
-    template <typename S, typename... TArgs>
+    template <typename... TArgs>
     void log(log_level with_level, char const *prefix,
-             fmt::text_style const &style, S const &format_str, TArgs &&...args)
+             fmt::text_style const &style,
+             fmt::format_string<TArgs...> format_str, TArgs &&...args)
     {
         if (with_level < m_current_level) {
             return;
@@ -109,37 +111,37 @@ private:
 
 logger &get_logger() noexcept;
 
-template <typename S, typename... TArgs>
-void log_debug(S const &format_str, TArgs &&... args)
+template <typename... TArgs>
+void log_debug(fmt::format_string<TArgs...> format_str, TArgs &&...args)
 {
     get_logger().log(log_level::debug, nullptr, {}, format_str,
                      std::forward<TArgs>(args)...);
 }
 
-template <typename S, typename... TArgs>
-void log_info(S const &format_str, TArgs &&... args)
+template <typename... TArgs>
+void log_info(fmt::format_string<TArgs...> format_str, TArgs &&...args)
 {
     get_logger().log(log_level::info, nullptr, {}, format_str,
                      std::forward<TArgs>(args)...);
 }
 
-template <typename S, typename... TArgs>
-void log_warn(S const &format_str, TArgs &&... args)
+template <typename... TArgs>
+void log_warn(fmt::format_string<TArgs...> format_str, TArgs &&...args)
 {
     get_logger().log(log_level::warn, "WARNING", fg(fmt::color::red),
                      format_str, std::forward<TArgs>(args)...);
 }
 
-template <typename S, typename... TArgs>
-void log_error(S const &format_str, TArgs &&... args)
+template <typename... TArgs>
+void log_error(fmt::format_string<TArgs...> format_str, TArgs &&...args)
 {
     get_logger().log(log_level::error, "ERROR",
                      fmt::emphasis::bold | fg(fmt::color::red), format_str,
                      std::forward<TArgs>(args)...);
 }
 
-template <typename S, typename... TArgs>
-void log_sql(S const &format_str, TArgs &&... args)
+template <typename... TArgs>
+void log_sql(fmt::format_string<TArgs...> format_str, TArgs &&...args)
 {
     auto &logger = get_logger();
     if (logger.log_sql()) {
@@ -148,8 +150,8 @@ void log_sql(S const &format_str, TArgs &&... args)
     }
 }
 
-template <typename S, typename... TArgs>
-void log_sql_data(S const &format_str, TArgs &&... args)
+template <typename... TArgs>
+void log_sql_data(fmt::format_string<TArgs...> format_str, TArgs &&...args)
 {
     auto &logger = get_logger();
     if (logger.log_sql_data()) {
