@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -12,7 +12,6 @@
 #include "format.hpp"
 
 #include <cassert>
-#include <stdexcept>
 
 // The lua_getextraspace() function is only available from Lua 5.3. For
 // earlier versions we fall back to storing the context pointer in the
@@ -34,8 +33,12 @@ void *luaX_get_context(lua_State *lua_state) noexcept
 
 #else
 
+namespace {
+
 // Unique key for lua registry
-static char const *osm2pgsql_output_flex = "osm2pgsql_output_flex";
+constexpr char const *const osm2pgsql_output_flex = "osm2pgsql_output_flex";
+
+} // anonymous namespace
 
 void luaX_set_context(lua_State *lua_state, void *ptr) noexcept
 {
@@ -59,6 +62,11 @@ void *luaX_get_context(lua_State *lua_state) noexcept
 
 #endif
 
+void luaX_pushstring(lua_State *lua_state, std::string_view str) noexcept
+{
+    lua_pushlstring(lua_state, str.data(), str.size());
+}
+
 void luaX_add_table_str(lua_State *lua_state, char const *key,
                         char const *value) noexcept
 {
@@ -193,7 +201,9 @@ int luaX_pcall(lua_State *lua_state, int narg, int nres)
 
 #else
 
-static int pcall_error_traceback_handler(lua_State *lua_state)
+namespace {
+
+int pcall_error_traceback_handler(lua_State *lua_state)
 {
     assert(lua_state);
 
@@ -210,6 +220,8 @@ static int pcall_error_traceback_handler(lua_State *lua_state)
     return 1;
 }
 
+} // anonymous namespace
+
 /// Wrapper function for lua_pcall() showing a stack trace on error.
 int luaX_pcall(lua_State *lua_state, int narg, int nres)
 {
