@@ -33,6 +33,7 @@
 #include <nlohmann/json.hpp>
 
 #include "format.hpp"
+#include "idlist.hpp"
 #include "json-writer.hpp"
 #include "logging.hpp"
 #include "middle-pgsql.hpp"
@@ -43,20 +44,10 @@
 #include "pgsql-helper.hpp"
 #include "util.hpp"
 
-static bool check_bucket_index(pg_conn_t const *db_connection,
-                               std::string const &prefix)
-{
-    auto const res =
-        db_connection->exec("SELECT relname FROM pg_class"
-                            " WHERE relkind='i'"
-                            " AND relname = '{}_ways_nodes_bucket_idx'",
-                            prefix);
-    return res.num_tuples() > 0;
-}
+namespace {
 
-static void send_id_list(pg_conn_t const &db_connection,
-                         std::string const &table,
-                         osmium::index::IdSetSmall<osmid_t> const &ids)
+void send_id_list(pg_conn_t const &db_connection,
+                         std::string const &table, idlist_t const &ids)
 {
     std::string data;
     for (auto const id : ids) {
@@ -69,18 +60,17 @@ static void send_id_list(pg_conn_t const &db_connection,
     db_connection.copy_end(table);
 }
 
-static void load_id_list(pg_conn_t const &db_connection,
-                         std::string const &table,
-                         osmium::index::IdSetSmall<osmid_t> *ids)
+void load_id_list(pg_conn_t const &db_connection, std::string const &table,
+                  idlist_t *ids)
 {
     auto const res = db_connection.exec(
         fmt::format("SELECT DISTINCT id FROM {} ORDER BY id", table));
     for (int n = 0; n < res.num_tuples(); ++n) {
-        ids->set(osmium::string_to_object_id(res.get_value(n, 0)));
+        ids->push_back(osmium::string_to_object_id(res.get_value(n, 0)));
     }
 }
 
-static std::string build_sql(options_t const &options, std::string const &templ)
+std::string build_sql(options_t const &options, std::string const &templ)
 {
     std::string const using_tablespace{options.tblsslim_index.empty()
                                            ? ""
@@ -96,7 +86,7 @@ static std::string build_sql(options_t const &options, std::string const &templ)
         fmt::arg("using_tablespace", using_tablespace),
         fmt::arg("data_tablespace", tablespace_clause(options.tblsslim_data)),
         fmt::arg("index_tablespace", tablespace_clause(options.tblsslim_index)),
-        fmt::arg("way_node_index_id_shift", options.way_node_index_id_shift),
+        fmt::arg("way_node_index_id_shift", 5),
         fmt::arg("attribute_columns_definition",
                  options.extra_attributes ? " created timestamp with time zone,"
                                             " version int4,"
@@ -115,8 +105,8 @@ static std::string build_sql(options_t const &options, std::string const &templ)
                      : ""));
 }
 
-static std::vector<std::string>
-build_sql(options_t const &options, std::vector<std::string> const &templs)
+std::vector<std::string> build_sql(options_t const &options,
+                                   std::vector<std::string> const &templs)
 {
     std::vector<std::string> out;
     out.reserve(templs.size());
@@ -128,6 +118,8 @@ build_sql(options_t const &options, std::vector<std::string> const &templs)
     return out;
 }
 
+} // anonymous namespace
+
 middle_pgsql_t::table_desc::table_desc(options_t const &options,
                                        table_sql const &ts)
 : m_create_table(build_sql(options, ts.create_table)),
@@ -135,14 +127,12 @@ middle_pgsql_t::table_desc::table_desc(options_t const &options,
   m_copy_target(std::make_shared<db_target_descr_t>(
       options.middle_dbschema, build_sql(options, ts.name), "id"))
 {
-    if (options.with_forward_dependencies) {
-        m_create_fw_dep_indexes = build_sql(options, ts.create_fw_dep_indexes);
-    }
+    m_create_fw_dep_indexes = build_sql(options, ts.create_fw_dep_indexes);
 }
 
 void middle_query_pgsql_t::exec_sql(std::string const &sql_cmd) const
 {
-    m_sql_conn.exec(sql_cmd);
+    m_db_connection.exec(sql_cmd);
 }
 
 void middle_pgsql_t::table_desc::drop_table(
@@ -151,10 +141,7 @@ void middle_pgsql_t::table_desc::drop_table(
     util::timer_t timer;
 
     log_info("Dropping table '{}'", name());
-
-    auto const qual_name = qualified_name(schema(), name());
-    db_connection.exec("DROP TABLE IF EXISTS {}", qual_name);
-
+    drop_table_if_exists(db_connection, schema(), name());
     log_info("Table '{}' dropped in {}", name(),
              util::human_readable_duration(timer.stop()));
 }
@@ -176,6 +163,14 @@ void middle_pgsql_t::table_desc::build_index(
     }
 }
 
+void middle_pgsql_t::table_desc::create_table(
+    pg_conn_t const &db_connection) const
+{
+    if (!m_create_table.empty()) {
+        db_connection.exec(m_create_table);
+    }
+}
+
 void middle_pgsql_t::table_desc::init_max_id(pg_conn_t const &db_connection)
 {
     auto const qual_name = qualified_name(schema(), name());
@@ -188,134 +183,8 @@ void middle_pgsql_t::table_desc::init_max_id(pg_conn_t const &db_connection)
     m_max_id = osmium::string_to_object_id(res.get_value(0, 0));
 }
 
-/**
- * Decode item in an array literal from PostgreSQL to the next delimiter.
- *
- * \param src Pointer to the text with the array literal.
- * \param dst The decoded item is written to this string. The string is
- *            cleared before use.
- * \returns Pointer to the delimiter found.
- * \throws runtime_error If the input string ends before the delimiter is found.
- */
-static char const *decode_to_delimiter(char const *src, std::string *dst)
-{
-    assert(src);
-    dst->clear();
-
-    bool const quoted = (*src == '"');
-    if (quoted) {
-        ++src;
-    }
-
-    while (quoted ? (*src != '"') : (*src != ',' && *src != '}')) {
-        if (*src == '\0') {
-            throw std::runtime_error{
-                "Parsing array literal from database failed."};
-        }
-        if (*src == '\\') {
-            switch (src[1]) {
-            case 'n':
-                dst->append(1, '\n');
-                break;
-            case 't':
-                dst->append(1, '\t');
-                break;
-            default:
-                dst->append(1, src[1]);
-                break;
-            }
-            src += 2;
-        } else {
-            dst->append(1, *src++);
-        }
-    }
-
-    if (quoted) {
-        ++src;
-    }
-
-    return src;
-}
-
 namespace {
 
-/**
- * Go through tags returned from a version 1 middle table, get the attributes
- * from the "osm_*" tags and add them to the builder.
- */
-template <typename T>
-void pgsql_get_attr_from_tags(char const *string, T *builder)
-{
-    if (*string++ != '{') {
-        return;
-    }
-
-    std::string key;
-    std::string val;
-    std::string user;
-    while (*string != '}') {
-        string = decode_to_delimiter(string, &key);
-        // String points to the comma */
-        ++string;
-        string = decode_to_delimiter(string, &val);
-
-        if (key == "osm_version") {
-            builder->set_version(val.c_str());
-        } else if (key == "osm_timestamp") {
-            builder->set_timestamp(osmium::Timestamp{val});
-        } else if (key == "osm_changeset") {
-            builder->set_changeset(val.c_str());
-        } else if (key == "osm_uid") {
-            builder->set_uid(val.c_str());
-        } else if (key == "osm_user") {
-            user = val;
-        }
-
-        // String points to the comma or closing '}' */
-        if (*string == ',') {
-            ++string;
-        }
-    }
-
-    // Must be done at the end, because after that call the builder might
-    // become invalid due to buffer resizing.
-    builder->set_user(user);
-}
-
-/**
- * Go through tags returned from a version 1 middle table, get all tags except
- * the pseudo-tags containing attributes and add them to the builder.
- */
-template <typename T>
-void pgsql_parse_tags(char const *string, osmium::memory::Buffer *buffer,
-                      T *obuilder)
-{
-    if (*string++ != '{') {
-        return;
-    }
-
-    osmium::builder::TagListBuilder builder{*buffer, obuilder};
-
-    std::string key;
-    std::string val;
-    while (*string != '}') {
-        string = decode_to_delimiter(string, &key);
-        // String points to the comma */
-        ++string;
-        string = decode_to_delimiter(string, &val);
-
-        if (key != "osm_version" && key != "osm_timestamp" &&
-            key != "osm_changeset" && key != "osm_uid" && key != "osm_user") {
-            builder.add_tag(key, val);
-        }
-
-        // String points to the comma or closing '}' */
-        if (*string == ',') {
-            ++string;
-        }
-    }
-}
-
 /**
  * Parse JSON-encoded tags from a version 2 middle table and add them to the
  * builder.
@@ -453,30 +322,6 @@ void pgsql_parse_json_members(char const *string,
     nlohmann::json::sax_parse(string, &parser);
 }
 
-void pgsql_parse_members(char const *string, osmium::memory::Buffer *buffer,
-                         osmium::builder::RelationBuilder *obuilder)
-{
-    if (*string++ != '{') {
-        return;
-    }
-
-    std::string role;
-    osmium::builder::RelationMemberListBuilder builder{*buffer, obuilder};
-
-    while (*string != '}') {
-        char const type = string[0];
-        char *endp = nullptr;
-        osmid_t const id = std::strtoll(string + 1, &endp, 10);
-        // String points to the comma */
-        string = decode_to_delimiter(endp + 1, &role);
-        builder.add_member(osmium::char_to_item_type(type), id, role);
-        // String points to the comma or closing '}' */
-        if (*string == ',') {
-            ++string;
-        }
-    }
-}
-
 void pgsql_parse_nodes(char const *string, osmium::memory::Buffer *buffer,
                        osmium::builder::WayBuilder *obuilder)
 {
@@ -515,9 +360,7 @@ void set_attributes_on_builder(T *builder, pg_result_t const &result, int num,
     }
 }
 
-} // anonymous namespace
-
-static void tags_to_json(osmium::TagList const &tags, json_writer_t *writer)
+void tags_to_json(osmium::TagList const &tags, json_writer_t *writer)
 {
     writer->start_object();
 
@@ -530,8 +373,8 @@ static void tags_to_json(osmium::TagList const &tags, json_writer_t *writer)
     writer->end_object();
 }
 
-static void members_to_json(osmium::RelationMemberList const &members,
-                            json_writer_t *writer)
+void members_to_json(osmium::RelationMemberList const &members,
+                     json_writer_t *writer)
 {
     writer->start_array();
 
@@ -566,30 +409,7 @@ static void members_to_json(osmium::RelationMemberList const &members,
     writer->end_array();
 }
 
-void middle_pgsql_t::buffer_store_tags(osmium::OSMObject const &obj, bool attrs)
-{
-    if (obj.tags().empty() && !attrs) {
-        m_db_copy.add_null_column();
-    } else {
-        m_db_copy.new_array();
-
-        for (auto const &it : obj.tags()) {
-            m_db_copy.add_array_elem(it.key());
-            m_db_copy.add_array_elem(it.value());
-        }
-
-        if (attrs) {
-            taglist_t extra;
-            extra.add_attributes(obj);
-            for (auto const &it : extra) {
-                m_db_copy.add_array_elem(it.key);
-                m_db_copy.add_array_elem(it.value);
-            }
-        }
-
-        m_db_copy.finish_array();
-    }
-}
+} // anonymous namespace
 
 void middle_pgsql_t::copy_attributes(osmium::OSMObject const &obj)
 {
@@ -621,17 +441,13 @@ void middle_pgsql_t::copy_attributes(osmium::OSMObject const &obj)
 
 void middle_pgsql_t::copy_tags(osmium::OSMObject const &obj)
 {
-    if (m_store_options.db_format == 2) {
-        if (obj.tags().empty()) {
-            m_db_copy.add_null_column();
-            return;
-        }
-        json_writer_t writer;
-        tags_to_json(obj.tags(), &writer);
-        m_db_copy.add_column(writer.json());
+    if (obj.tags().empty()) {
+        m_db_copy.add_null_column();
         return;
     }
-    buffer_store_tags(obj, m_store_options.with_attributes);
+    json_writer_t writer;
+    tags_to_json(obj.tags(), &writer);
+    m_db_copy.add_column(writer.json());
 }
 
 std::size_t middle_query_pgsql_t::get_way_node_locations_db(
@@ -658,7 +474,7 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_db(
 
     // get any remaining nodes from the DB
     // Nodes must have been written back at this point.
-    auto const res = m_sql_conn.exec_prepared("get_node_list", id_list());
+    auto const res = m_db_connection.exec_prepared("get_node_list", id_list());
     std::unordered_map<osmid_t, osmium::Location> locs;
     for (int i = 0; i < res.num_tuples(); ++i) {
         locs.emplace(osmium::string_to_object_id(res.get_value(i, 0)),
@@ -738,12 +554,10 @@ void middle_pgsql_t::node_set(osmium::Node const &node)
 
     m_db_copy.add_columns(node.id(), node.location().y(), node.location().x());
 
-    if (m_store_options.db_format == 2) {
-        if (m_store_options.with_attributes) {
-            copy_attributes(node);
-        }
-        copy_tags(node);
+    if (m_store_options.with_attributes) {
+        copy_attributes(node);
     }
+    copy_tags(node);
 
     m_db_copy.finish_line();
 }
@@ -769,7 +583,7 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_flatnodes(
 
 osmium::Location middle_query_pgsql_t::get_node_location_db(osmid_t id) const
 {
-    auto const res = m_sql_conn.exec_prepared("get_node", id);
+    auto const res = m_db_connection.exec_prepared("get_node", id);
     if (res.num_tuples() == 0) {
         return osmium::Location{};
     }
@@ -818,10 +632,9 @@ void middle_pgsql_t::node_delete(osmid_t osm_id)
     }
 }
 
-void middle_pgsql_t::get_node_parents(
-    osmium::index::IdSetSmall<osmid_t> const &changed_nodes,
-    osmium::index::IdSetSmall<osmid_t> *parent_ways,
-    osmium::index::IdSetSmall<osmid_t> *parent_relations) const
+void middle_pgsql_t::get_node_parents(idlist_t const &changed_nodes,
+                                      idlist_t *parent_ways,
+                                      idlist_t *parent_relations) const
 {
     util::timer_t timer;
 
@@ -839,18 +652,14 @@ void middle_pgsql_t::get_node_parents(
 
     queries.emplace_back("ANALYZE osm2pgsql_changed_nodes");
 
-    bool const has_bucket_index =
-        check_bucket_index(&m_db_connection, m_options->prefix);
-
-    if (has_bucket_index) {
-        // The query to get the parent ways of changed nodes is "hidden"
-        // inside a PL/pgSQL function so that the query planner only sees
-        // a single node id that is being queried for. If we ask for all
-        // nodes at the same time the query planner sometimes thinks it is
-        // better to do a full table scan which totally destroys performance.
-        // This is due to the PostgreSQL statistics on ARRAYs being way off.
-        queries.emplace_back(R"(
-CREATE OR REPLACE FUNCTION osm2pgsql_find_changed_ways() RETURNS void AS $$
+    // The query to get the parent ways of changed nodes is "hidden"
+    // inside a PL/pgSQL function so that the query planner only sees
+    // a single node id that is being queried for. If we ask for all
+    // nodes at the same time the query planner sometimes thinks it is
+    // better to do a full table scan which totally destroys performance.
+    // This is due to the PostgreSQL statistics on ARRAYs being way off.
+    queries.emplace_back(R"(
+CREATE OR REPLACE FUNCTION {schema}osm2pgsql_find_changed_ways() RETURNS void AS $$
 DECLARE
   changed_buckets RECORD;
 BEGIN
@@ -868,39 +677,24 @@ BEGIN
 END;
 $$ LANGUAGE plpgsql
 )");
-        queries.emplace_back("SELECT osm2pgsql_find_changed_ways()");
-        queries.emplace_back("DROP FUNCTION osm2pgsql_find_changed_ways()");
-    } else {
-        queries.emplace_back(R"(
-INSERT INTO osm2pgsql_changed_ways
-  SELECT w.id
-    FROM {schema}"{prefix}_ways" w, osm2pgsql_changed_nodes n
-    WHERE w.nodes && ARRAY[n.id]
-        )");
-    }
+    queries.emplace_back("SELECT {schema}osm2pgsql_find_changed_ways()");
+    queries.emplace_back("DROP FUNCTION {schema}osm2pgsql_find_changed_ways()");
 
-    if (m_options->middle_database_format == 1) {
-        queries.emplace_back(R"(
-INSERT INTO osm2pgsql_changed_relations
-  SELECT r.id
-    FROM {schema}"{prefix}_rels" r, osm2pgsql_changed_nodes n
-    WHERE r.parts && ARRAY[n.id]
-      AND r.parts[1:way_off] && ARRAY[n.id]
-        )");
-    } else {
-        queries.emplace_back(R"(
+    queries.emplace_back(R"(
 INSERT INTO osm2pgsql_changed_relations
   SELECT r.id
     FROM {schema}"{prefix}_rels" r, osm2pgsql_changed_nodes c
     WHERE {schema}"{prefix}_member_ids"(r.members, 'N'::char) && ARRAY[c.id];
-        )");
-    }
+    )");
 
     for (auto const &query : queries) {
         m_db_connection.exec(build_sql(*m_options, query));
     }
 
-    load_id_list(m_db_connection, "osm2pgsql_changed_ways", parent_ways);
+    if (parent_ways) {
+        load_id_list(m_db_connection, "osm2pgsql_changed_ways", parent_ways);
+    }
+
     load_id_list(m_db_connection, "osm2pgsql_changed_relations",
                  parent_relations);
 
@@ -909,14 +703,21 @@ INSERT INTO osm2pgsql_changed_relations
     timer.stop();
 
     log_debug("Found {} new/changed nodes in input.", changed_nodes.size());
-    log_debug("  Found in {} their {} parent ways and {} parent relations.",
-              std::chrono::duration_cast<std::chrono::seconds>(timer.elapsed()),
-              parent_ways->size(), parent_relations->size());
+
+    auto const elapsed_sec =
+        std::chrono::duration_cast<std::chrono::seconds>(timer.elapsed());
+
+    if (parent_ways) {
+        log_debug("  Found in {} their {} parent ways and {} parent relations.",
+                  elapsed_sec, parent_ways->size(), parent_relations->size());
+    } else {
+        log_debug("  Found in {} their {} parent relations.", elapsed_sec,
+                  parent_relations->size());
+    }
 }
 
-void middle_pgsql_t::get_way_parents(
-    osmium::index::IdSetSmall<osmid_t> const &changed_ways,
-    osmium::index::IdSetSmall<osmid_t> *parent_relations) const
+void middle_pgsql_t::get_way_parents(idlist_t const &changed_ways,
+                                     idlist_t *parent_relations) const
 {
     util::timer_t timer;
 
@@ -932,22 +733,12 @@ void middle_pgsql_t::get_way_parents(
 
     m_db_connection.exec("ANALYZE osm2pgsql_changed_ways");
 
-    if (m_options->middle_database_format == 1) {
-        m_db_connection.exec(build_sql(*m_options, R"(
-INSERT INTO osm2pgsql_changed_relations
-  SELECT DISTINCT r.id
-    FROM {schema}"{prefix}_rels" r, osm2pgsql_changed_ways w
-    WHERE r.parts && ARRAY[w.id]
-      AND r.parts[way_off+1:rel_off] && ARRAY[w.id]
-        )"));
-    } else {
-        m_db_connection.exec(build_sql(*m_options, R"(
+    m_db_connection.exec(build_sql(*m_options, R"(
 INSERT INTO osm2pgsql_changed_relations
   SELECT DISTINCT r.id
     FROM {schema}"{prefix}_rels" r, osm2pgsql_changed_ways c
     WHERE {schema}"{prefix}_member_ids"(r.members, 'W'::char) && ARRAY[c.id];
-        )"));
-    }
+    )"));
 
     load_id_list(m_db_connection, "osm2pgsql_changed_relations",
                  parent_relations);
@@ -973,7 +764,7 @@ void middle_pgsql_t::way_set(osmium::Way const &way)
 
     m_db_copy.add_column(way.id());
 
-    if (m_store_options.db_format == 2 && m_store_options.with_attributes) {
+    if (m_store_options.with_attributes) {
         copy_attributes(way);
     }
 
@@ -989,26 +780,32 @@ void middle_pgsql_t::way_set(osmium::Way const &way)
     m_db_copy.finish_line();
 }
 
+namespace {
+
+/**
+ * Build node in buffer from database results.
+ */
+void build_node(osmid_t id, pg_result_t const &res, int res_num, int offset,
+                osmium::memory::Buffer *buffer, bool with_attributes)
+{
+    osmium::builder::NodeBuilder builder{*buffer};
+    builder.set_id(id);
+
+    if (with_attributes) {
+        set_attributes_on_builder(&builder, res, res_num, offset);
+    }
+    pgsql_parse_json_tags(res.get_value(res_num, offset + 1), buffer, &builder);
+}
+
 /**
  * Build way in buffer from database results.
  */
-static void build_way(osmid_t id, pg_result_t const &res, int res_num,
-                      int offset, osmium::memory::Buffer *buffer,
-                      uint8_t db_format, bool with_attributes)
+void build_way(osmid_t id, pg_result_t const &res, int res_num, int offset,
+               osmium::memory::Buffer *buffer, bool with_attributes)
 {
     osmium::builder::WayBuilder builder{*buffer};
     builder.set_id(id);
 
-    if (db_format == 1) {
-        if (with_attributes) {
-            pgsql_get_attr_from_tags(res.get_value(res_num, offset + 1),
-                                     &builder);
-        }
-        pgsql_parse_nodes(res.get_value(res_num, offset + 0), buffer, &builder);
-        pgsql_parse_tags(res.get_value(res_num, offset + 1), buffer, &builder);
-        return;
-    }
-
     if (with_attributes) {
         set_attributes_on_builder(&builder, res, res_num, offset);
     }
@@ -1016,19 +813,38 @@ static void build_way(osmid_t id, pg_result_t const &res, int res_num,
     pgsql_parse_json_tags(res.get_value(res_num, offset + 1), buffer, &builder);
 }
 
-bool middle_query_pgsql_t::way_get(osmid_t id,
-                                   osmium::memory::Buffer *buffer) const
+} // anonymous namespace
+
+bool middle_query_pgsql_t::node_get(osmid_t id,
+                                    osmium::memory::Buffer *buffer) const
 {
     assert(buffer);
 
-    auto const res = m_sql_conn.exec_prepared("get_way", id);
+    auto const res = m_db_connection.exec_prepared("get_node", id);
 
     if (res.num_tuples() != 1) {
         return false;
     }
 
-    build_way(id, res, 0, 0, buffer, m_store_options.db_format,
-              m_store_options.with_attributes);
+    build_node(id, res, 0, 0, buffer, m_store_options.with_attributes);
+
+    buffer->commit();
+
+    return true;
+}
+
+bool middle_query_pgsql_t::way_get(osmid_t id,
+                                   osmium::memory::Buffer *buffer) const
+{
+    assert(buffer);
+
+    auto const res = m_db_connection.exec_prepared("get_way", id);
+
+    if (res.num_tuples() != 1) {
+        return false;
+    }
+
+    build_way(id, res, 0, 0, buffer, m_store_options.with_attributes);
 
     buffer->commit();
 
@@ -1056,7 +872,7 @@ middle_query_pgsql_t::rel_members_get(osmium::Relation const &rel,
 
         // ...and get those ways from database
         if (!way_ids.empty()) {
-            res = m_sql_conn.exec_prepared("get_way_list", way_ids());
+            res = m_db_connection.exec_prepared("get_way_list", way_ids());
             wayidspg = get_ids_from_result(res);
         }
     }
@@ -1075,7 +891,6 @@ middle_query_pgsql_t::rel_members_get(osmium::Relation const &rel,
             for (int j = 0; j < res.num_tuples(); ++j) {
                 if (member.ref() == wayidspg[static_cast<std::size_t>(j)]) {
                     build_way(member.ref(), res, j, 1, buffer,
-                              m_store_options.db_format,
                               m_store_options.with_attributes);
                     ++members_found;
                     break;
@@ -1099,50 +914,7 @@ void middle_pgsql_t::way_delete(osmid_t osm_id)
     }
 }
 
-void middle_pgsql_t::relation_set_format1(osmium::Relation const &rel)
-{
-    // Sort relation members by their type.
-    std::array<idlist_t, 3> parts;
-
-    for (auto const &m : rel.members()) {
-        parts.at(osmium::item_type_to_nwr_index(m.type())).push_back(m.ref());
-    }
-
-    m_db_copy.new_line(m_tables.relations().copy_target());
-
-    // id, way offset, relation offset
-    m_db_copy.add_columns(rel.id(), parts[0].size(),
-                          parts[0].size() + parts[1].size());
-
-    // parts
-    m_db_copy.new_array();
-    for (auto const &part : parts) {
-        for (auto it : part) {
-            m_db_copy.add_array_elem(it);
-        }
-    }
-    m_db_copy.finish_array();
-
-    // members
-    if (rel.members().empty()) {
-        m_db_copy.add_null_column();
-    } else {
-        m_db_copy.new_array();
-        for (auto const &m : rel.members()) {
-            m_db_copy.add_array_elem(osmium::item_type_to_char(m.type()) +
-                                     std::to_string(m.ref()));
-            m_db_copy.add_array_elem(m.role());
-        }
-        m_db_copy.finish_array();
-    }
-
-    // tags
-    buffer_store_tags(rel, m_options->extra_attributes);
-
-    m_db_copy.finish_line();
-}
-
-void middle_pgsql_t::relation_set_format2(osmium::Relation const &rel)
+void middle_pgsql_t::relation_set(osmium::Relation const &rel)
 {
     m_db_copy.new_line(m_tables.relations().copy_target());
     m_db_copy.add_column(rel.id());
@@ -1160,47 +932,12 @@ void middle_pgsql_t::relation_set_format2(osmium::Relation const &rel)
     m_db_copy.finish_line();
 }
 
-void middle_pgsql_t::relation_set(osmium::Relation const &rel)
-{
-    if (m_store_options.db_format == 2) {
-        return relation_set_format2(rel);
-    }
-    return relation_set_format1(rel);
-}
-
-bool middle_query_pgsql_t::relation_get_format1(
-    osmid_t id, osmium::memory::Buffer *buffer) const
+bool middle_query_pgsql_t::relation_get(osmid_t id,
+                                        osmium::memory::Buffer *buffer) const
 {
     assert(buffer);
 
-    auto const res = m_sql_conn.exec_prepared("get_rel", id);
-    if (res.num_tuples() != 1) {
-        return false;
-    }
-
-    {
-        osmium::builder::RelationBuilder builder{*buffer};
-        builder.set_id(id);
-        if (m_store_options.with_attributes) {
-            pgsql_get_attr_from_tags(res.get_value(0, 1), &builder);
-        }
-
-        pgsql_get_attr_from_tags(res.get_value(0, 1), &builder);
-        pgsql_parse_members(res.get_value(0, 0), buffer, &builder);
-        pgsql_parse_tags(res.get_value(0, 1), buffer, &builder);
-    }
-
-    buffer->commit();
-
-    return true;
-}
-
-bool middle_query_pgsql_t::relation_get_format2(
-    osmid_t id, osmium::memory::Buffer *buffer) const
-{
-    assert(buffer);
-
-    auto const res = m_sql_conn.exec_prepared("get_rel", id);
+    auto const res = m_db_connection.exec_prepared("get_rel", id);
 
     if (res.num_tuples() == 0) {
         return false;
@@ -1222,15 +959,6 @@ bool middle_query_pgsql_t::relation_get_format2(
     return true;
 }
 
-bool middle_query_pgsql_t::relation_get(osmid_t id,
-                                        osmium::memory::Buffer *buffer) const
-{
-    if (m_store_options.db_format == 2) {
-        return relation_get_format2(id, buffer);
-    }
-    return relation_get_format1(id, buffer);
-}
-
 void middle_pgsql_t::relation_delete(osmid_t osm_id)
 {
     assert(m_options->append);
@@ -1253,6 +981,8 @@ void middle_pgsql_t::after_nodes()
         auto const &table = m_tables.nodes();
         analyze_table(m_db_connection, table.schema(), table.name());
     }
+
+    m_cache->log_stats();
 }
 
 void middle_pgsql_t::after_ways()
@@ -1282,8 +1012,7 @@ void middle_pgsql_t::after_relations()
         analyze_table(m_db_connection, table.schema(), table.name());
     }
 
-    if (m_store_options.db_format == 2 && m_store_options.with_attributes &&
-        !m_options->droptemp) {
+    if (m_store_options.with_attributes && !m_options->droptemp) {
         if (m_append) {
             update_users_table();
         } else {
@@ -1300,26 +1029,27 @@ middle_query_pgsql_t::middle_query_pgsql_t(
     std::shared_ptr<node_locations_t> cache,
     std::shared_ptr<node_persistent_cache> persistent_cache,
     middle_pgsql_options const &options)
-: m_sql_conn(connection_params, "middle.query"), m_cache(std::move(cache)),
+: m_db_connection(connection_params, "middle.query"), m_cache(std::move(cache)),
   m_persistent_cache(std::move(persistent_cache)), m_store_options(options)
 {
     // Disable JIT and parallel workers as they are known to cause
     // problems when accessing the intarrays.
-    m_sql_conn.set_config("jit_above_cost", "-1");
-    m_sql_conn.set_config("max_parallel_workers_per_gather", "0");
+    m_db_connection.set_config("jit_above_cost", "-1");
+    m_db_connection.set_config("max_parallel_workers_per_gather", "0");
 }
 
-static void table_setup(pg_conn_t const &db_connection,
-                        middle_pgsql_t::table_desc const &table)
+namespace {
+
+void table_setup(pg_conn_t const &db_connection,
+                 middle_pgsql_t::table_desc const &table)
 {
     log_debug("Setting up table '{}'", table.name());
-    auto const qual_name = qualified_name(table.schema(), table.name());
-    db_connection.exec("DROP TABLE IF EXISTS {} CASCADE", qual_name);
-    if (!table.m_create_table.empty()) {
-        db_connection.exec(table.m_create_table);
-    }
+    drop_table_if_exists(db_connection, table.schema(), table.name());
+    table.create_table(db_connection);
 }
 
+} // anonymous namespace
+
 void middle_pgsql_t::start()
 {
     assert(m_middle_state == middle_state::constructed);
@@ -1343,9 +1073,7 @@ void middle_pgsql_t::start()
         m_tables.ways().init_max_id(m_db_connection);
         m_tables.relations().init_max_id(m_db_connection);
     } else {
-        if (m_store_options.db_format == 2) {
-            table_setup(m_db_connection, m_users_table);
-        }
+        table_setup(m_db_connection, m_users_table);
         for (auto const &table : m_tables) {
             table_setup(m_db_connection, table);
         }
@@ -1423,7 +1151,9 @@ void middle_pgsql_t::wait()
     }
 }
 
-static table_sql sql_for_users(middle_pgsql_options const &store_options)
+namespace {
+
+table_sql sql_for_users(middle_pgsql_options const &store_options)
 {
     table_sql sql{};
 
@@ -1439,33 +1169,7 @@ static table_sql sql_for_users(middle_pgsql_options const &store_options)
     return sql;
 }
 
-static table_sql sql_for_nodes_format1(bool create_table)
-{
-    table_sql sql{};
-
-    sql.name = "{prefix}_nodes";
-
-    if (create_table) {
-        sql.create_table =
-            "CREATE {unlogged} TABLE {schema}\"{prefix}_nodes\" ("
-            "  id int8 PRIMARY KEY {using_tablespace},"
-            "  lat int4 NOT NULL,"
-            "  lon int4 NOT NULL"
-            ") {data_tablespace}";
-
-        sql.prepare_queries = {
-            "PREPARE get_node_list(int8[]) AS"
-            "  SELECT id, lon, lat FROM {schema}\"{prefix}_nodes\""
-            "  WHERE id = ANY($1::int8[])",
-            "PREPARE get_node(int8) AS"
-            "  SELECT id, lon, lat FROM {schema}\"{prefix}_nodes\""
-            "  WHERE id = $1"};
-    }
-
-    return sql;
-}
-
-static table_sql sql_for_nodes_format2(middle_pgsql_options const &options)
+table_sql sql_for_nodes(middle_pgsql_options const &options)
 {
     table_sql sql{};
 
@@ -1493,48 +1197,7 @@ static table_sql sql_for_nodes_format2(middle_pgsql_options const &options)
     return sql;
 }
 
-static table_sql sql_for_ways_format1(uint8_t way_node_index_id_shift)
-{
-    table_sql sql{};
-
-    sql.name = "{prefix}_ways";
-
-    sql.create_table = "CREATE {unlogged} TABLE {schema}\"{prefix}_ways\" ("
-                       "  id int8 PRIMARY KEY {using_tablespace},"
-                       "  nodes int8[] NOT NULL,"
-                       "  tags text[]"
-                       ") {data_tablespace}";
-
-    sql.prepare_queries = {"PREPARE get_way(int8) AS"
-                           "  SELECT nodes, tags"
-                           "    FROM {schema}\"{prefix}_ways\" WHERE id = $1",
-                           "PREPARE get_way_list(int8[]) AS"
-                           "  SELECT id, nodes, tags"
-                           "    FROM {schema}\"{prefix}_ways\""
-                           "      WHERE id = ANY($1::int8[])"};
-
-    if (way_node_index_id_shift == 0) {
-        sql.create_fw_dep_indexes = {
-            "CREATE INDEX ON {schema}\"{prefix}_ways\" USING GIN (nodes)"
-            "  WITH (fastupdate = off) {index_tablespace}"};
-    } else {
-        sql.create_fw_dep_indexes = {
-            "CREATE OR REPLACE FUNCTION"
-            "    {schema}\"{prefix}_index_bucket\"(int8[])"
-            "  RETURNS int8[] AS $$"
-            "  SELECT ARRAY(SELECT DISTINCT"
-            "    unnest($1) >> {way_node_index_id_shift})"
-            "$$ LANGUAGE SQL IMMUTABLE",
-            "CREATE INDEX \"{prefix}_ways_nodes_bucket_idx\""
-            "  ON {schema}\"{prefix}_ways\""
-            "  USING GIN ({schema}\"{prefix}_index_bucket\"(nodes))"
-            "  WITH (fastupdate = off) {index_tablespace}"};
-    }
-
-    return sql;
-}
-
-static table_sql sql_for_ways_format2(middle_pgsql_options const &options)
+table_sql sql_for_ways()
 {
     table_sql sql{};
 
@@ -1558,54 +1221,22 @@ static table_sql sql_for_ways_format2(middle_pgsql_options const &options)
                            "  {users_table_access}"
                            " WHERE o.id = ANY($1::int8[])"};
 
-    if (options.way_node_index_id_shift == 0) {
-        sql.create_fw_dep_indexes = {
-            "CREATE INDEX ON {schema}\"{prefix}_ways\" USING GIN (nodes)"
-            "  WITH (fastupdate = off) {index_tablespace}"};
-    } else {
-        sql.create_fw_dep_indexes = {
-            "CREATE OR REPLACE FUNCTION"
-            "    {schema}\"{prefix}_index_bucket\"(int8[])"
-            "  RETURNS int8[] AS $$"
-            "  SELECT ARRAY(SELECT DISTINCT"
-            "    unnest($1) >> {way_node_index_id_shift})"
-            "$$ LANGUAGE SQL IMMUTABLE",
-            "CREATE INDEX \"{prefix}_ways_nodes_bucket_idx\""
-            "  ON {schema}\"{prefix}_ways\""
-            "  USING GIN ({schema}\"{prefix}_index_bucket\"(nodes))"
-            "  WITH (fastupdate = off) {index_tablespace}"};
-    }
-
-    return sql;
-}
-
-static table_sql sql_for_relations_format1()
-{
-    table_sql sql{};
-
-    sql.name = "{prefix}_rels";
-
-    sql.create_table = "CREATE {unlogged} TABLE {schema}\"{prefix}_rels\" ("
-                       "  id int8 PRIMARY KEY {using_tablespace},"
-                       "  way_off int2,"
-                       "  rel_off int2,"
-                       "  parts int8[],"
-                       "  members text[],"
-                       "  tags text[]"
-                       ") {data_tablespace}";
-
-    sql.prepare_queries = {"PREPARE get_rel(int8) AS"
-                           "  SELECT members, tags"
-                           "    FROM {schema}\"{prefix}_rels\" WHERE id = $1"};
-
     sql.create_fw_dep_indexes = {
-        "CREATE INDEX ON {schema}\"{prefix}_rels\" USING GIN (parts)"
+        "CREATE OR REPLACE FUNCTION"
+        "    {schema}\"{prefix}_index_bucket\"(int8[])"
+        "  RETURNS int8[] AS $$"
+        "  SELECT ARRAY(SELECT DISTINCT"
+        "    unnest($1) >> {way_node_index_id_shift})"
+        "$$ LANGUAGE SQL IMMUTABLE",
+        "CREATE INDEX \"{prefix}_ways_nodes_bucket_idx\""
+        "  ON {schema}\"{prefix}_ways\""
+        "  USING GIN ({schema}\"{prefix}_index_bucket\"(nodes))"
         "  WITH (fastupdate = off) {index_tablespace}"};
 
     return sql;
 }
 
-static table_sql sql_for_relations_format2()
+table_sql sql_for_relations()
 {
     table_sql sql{};
 
@@ -1644,6 +1275,8 @@ static table_sql sql_for_relations_format2()
     return sql;
 }
 
+} // anonymous namespace
+
 middle_pgsql_t::middle_pgsql_t(std::shared_ptr<thread_pool_t> thread_pool,
                                options_t const *options)
 : middle_t(std::move(thread_pool)), m_options(options),
@@ -1654,10 +1287,8 @@ middle_pgsql_t::middle_pgsql_t(std::shared_ptr<thread_pool_t> thread_pool,
   m_db_copy(m_copy_thread), m_append(options->append)
 {
     m_store_options.with_attributes = options->extra_attributes;
-    m_store_options.db_format = options->middle_database_format;
-    m_store_options.way_node_index_id_shift = options->way_node_index_id_shift;
 
-    if (m_store_options.db_format == 2 && options->middle_with_nodes) {
+    if (options->middle_with_nodes) {
         m_store_options.nodes = true;
     }
 
@@ -1670,40 +1301,13 @@ middle_pgsql_t::middle_pgsql_t(std::shared_ptr<thread_pool_t> thread_pool,
             options->flat_node_file, options->droptemp);
     }
 
-    log_debug("Mid: pgsql, cache={}, db_format={}", options->cache,
-              options->middle_database_format);
+    log_debug("Mid: pgsql, cache={}", options->cache);
 
-    bool const has_bucket_index =
-        check_bucket_index(&m_db_connection, options->prefix);
+    m_tables.nodes() = table_desc{*options, sql_for_nodes(m_store_options)};
+    m_tables.ways() = table_desc{*options, sql_for_ways()};
+    m_tables.relations() = table_desc{*options, sql_for_relations()};
 
-    if (!has_bucket_index && options->append &&
-        options->with_forward_dependencies) {
-        log_debug("You don't have a bucket index. See manual for details.");
-    }
-
-    if (m_store_options.db_format == 1) {
-        m_tables.nodes() = table_desc{
-            *options,
-            sql_for_nodes_format1(!m_store_options.use_flat_node_file)};
-
-        m_tables.ways() = table_desc{
-            *options,
-            sql_for_ways_format1(m_store_options.way_node_index_id_shift)};
-
-        m_tables.relations() =
-            table_desc{*options, sql_for_relations_format1()};
-    } else {
-        m_tables.nodes() =
-            table_desc{*options, sql_for_nodes_format2(m_store_options)};
-
-        m_tables.ways() =
-            table_desc{*options, sql_for_ways_format2(m_store_options)};
-
-        m_tables.relations() =
-            table_desc{*options, sql_for_relations_format2()};
-
-        m_users_table = table_desc{*options, sql_for_users(m_store_options)};
-    }
+    m_users_table = table_desc{*options, sql_for_users(m_store_options)};
 }
 
 void middle_pgsql_t::set_requirements(
@@ -1712,10 +1316,7 @@ void middle_pgsql_t::set_requirements(
     log_debug("Middle 'pgsql' options:");
     log_debug("  nodes: {}", m_store_options.nodes);
     log_debug("  untagged_nodes: {}", m_store_options.untagged_nodes);
-    log_debug("  db_format: {}", m_store_options.db_format);
     log_debug("  use_flat_node_file: {}", m_store_options.use_flat_node_file);
-    log_debug("  way_node_index_id_shift: {}",
-              m_store_options.way_node_index_id_shift);
     log_debug("  with_attributes: {}", m_store_options.with_attributes);
 }
 
@@ -1730,7 +1331,7 @@ middle_pgsql_t::get_query_instance()
 
     // We use a connection per table to enable the use of COPY
     for (auto &table : m_tables) {
-        for (auto const &query : table.m_prepare_queries) {
+        for (auto const &query : table.prepare_queries()) {
             mid->exec_sql(query);
         }
     }
