@@ -24,6 +24,7 @@
 #include <osmium/index/nwr_array.hpp>
 
 #include "db-copy-mgr.hpp"
+#include "idlist.hpp"
 #include "middle.hpp"
 #include "pgsql.hpp"
 
@@ -38,12 +39,6 @@ struct middle_pgsql_options
     // Store untagged nodes also (set in addition to nodes=true).
     bool untagged_nodes = false;
 
-    // Bit shift used in way node index
-    uint8_t way_node_index_id_shift = 5;
-
-    // Database format (legacy = 1, new = 2)
-    uint8_t db_format = 1;
-
     // Use a flat node file
     bool use_flat_node_file = false;
 
@@ -64,15 +59,14 @@ public:
 
     size_t nodes_get_list(osmium::WayNodeList *nodes) const override;
 
+    bool node_get(osmid_t id, osmium::memory::Buffer *buffer) const override;
+
     bool way_get(osmid_t id, osmium::memory::Buffer *buffer) const override;
 
     size_t rel_members_get(osmium::Relation const &rel,
                            osmium::memory::Buffer *buffer,
                            osmium::osm_entity_bits::type types) const override;
 
-    bool relation_get_format1(osmid_t id, osmium::memory::Buffer *buffer) const;
-    bool relation_get_format2(osmid_t id, osmium::memory::Buffer *buffer) const;
-
     bool relation_get(osmid_t id,
                       osmium::memory::Buffer *buffer) const override;
 
@@ -84,7 +78,7 @@ private:
     std::size_t get_way_node_locations_flatnodes(osmium::WayNodeList *nodes) const;
     std::size_t get_way_node_locations_db(osmium::WayNodeList *nodes) const;
 
-    pg_conn_t m_sql_conn;
+    pg_conn_t m_db_connection;
     std::shared_ptr<node_locations_t> m_cache;
     std::shared_ptr<node_persistent_cache> m_persistent_cache;
 
@@ -117,14 +111,11 @@ struct middle_pgsql_t : public middle_t
     void after_ways() override;
     void after_relations() override;
 
-    void get_node_parents(
-        osmium::index::IdSetSmall<osmid_t> const &changed_nodes,
-        osmium::index::IdSetSmall<osmid_t> *parent_ways,
-        osmium::index::IdSetSmall<osmid_t> *parent_relations) const override;
+    void get_node_parents(idlist_t const &changed_nodes, idlist_t *parent_ways,
+                          idlist_t *parent_relations) const override;
 
-    void get_way_parents(
-        osmium::index::IdSetSmall<osmid_t> const &changed_ways,
-        osmium::index::IdSetSmall<osmid_t> *parent_relations) const override;
+    void get_way_parents(idlist_t const &changed_ways,
+                         idlist_t *parent_relations) const override;
 
     class table_desc
     {
@@ -153,10 +144,6 @@ struct middle_pgsql_t : public middle_t
         ///< Open a new database connection and build index on this table.
         void build_index(connection_params_t const &connection_params) const;
 
-        std::string m_create_table;
-        std::vector<std::string> m_prepare_queries;
-        std::vector<std::string> m_create_fw_dep_indexes;
-
         void task_set(std::future<std::chrono::microseconds> &&future)
         {
             m_task_result.set(std::move(future));
@@ -164,11 +151,21 @@ struct middle_pgsql_t : public middle_t
 
         std::chrono::microseconds task_wait() { return m_task_result.wait(); }
 
+        void create_table(pg_conn_t const &db_connection) const;
+
         void init_max_id(pg_conn_t const &db_connection);
 
         osmid_t max_id() const noexcept { return m_max_id; }
 
+        std::vector<std::string> const &prepare_queries() const noexcept
+        {
+            return m_prepare_queries;
+        }
+
     private:
+        std::string m_create_table;
+        std::vector<std::string> m_create_fw_dep_indexes;
+        std::vector<std::string> m_prepare_queries;
         std::shared_ptr<db_target_descr_t> m_copy_target;
         task_result_t m_task_result;
 
@@ -187,14 +184,9 @@ private:
     void way_set(osmium::Way const &way);
     void way_delete(osmid_t id);
 
-    void relation_set_format1(osmium::Relation const &rel);
-    void relation_set_format2(osmium::Relation const &rel);
-
     void relation_set(osmium::Relation const &rel);
     void relation_delete(osmid_t id);
 
-    void buffer_store_tags(osmium::OSMObject const &obj, bool attrs);
-
     void copy_attributes(osmium::OSMObject const &obj);
     void copy_tags(osmium::OSMObject const &obj);
 
