@@ -7,8 +7,10 @@
  * For a full list of authors see the git log.
  */
 
-#include "logging.hpp"
 #include "middle-ram.hpp"
+
+#include "logging.hpp"
+#include "node-persistent-cache.hpp"
 #include "options.hpp"
 #include "output-requirements.hpp"
 
@@ -25,6 +27,47 @@
 #include <cassert>
 #include <memory>
 
+namespace {
+
+void add_delta_encoded_way_node_list(std::string *data,
+                                     osmium::WayNodeList const &wnl)
+{
+    assert(data);
+
+    // Add number of nodes in list
+    protozero::add_varint_to_buffer(data, wnl.size());
+
+    // Add delta encoded node ids
+    osmium::DeltaEncode<osmid_t> delta;
+    for (auto const &nr : wnl) {
+        protozero::add_varint_to_buffer(
+            data, protozero::encode_zigzag64(delta.update(nr.ref())));
+    }
+}
+
+void get_delta_encoded_way_nodes_list(std::string const &data,
+                                      std::size_t offset,
+                                      osmium::builder::WayBuilder *builder)
+{
+    assert(builder);
+
+    char const *begin = data.data() + offset;
+    char const *const end = data.data() + data.size();
+
+    auto count = protozero::decode_varint(&begin, end);
+
+    osmium::DeltaDecode<osmid_t> delta;
+    osmium::builder::WayNodeListBuilder wnl_builder{*builder};
+    while (count > 0) {
+        auto const val =
+            protozero::decode_zigzag64(protozero::decode_varint(&begin, end));
+        wnl_builder.add_node_ref(delta.update(val));
+        --count;
+    }
+}
+
+} // anonymous namespace
+
 middle_ram_t::middle_ram_t(std::shared_ptr<thread_pool_t> thread_pool,
                            options_t const *options)
 : middle_t(std::move(thread_pool))
@@ -34,6 +77,11 @@ middle_ram_t::middle_ram_t(std::shared_ptr<thread_pool_t> thread_pool,
     if (options->extra_attributes) {
         m_store_options.untagged_nodes = true;
     }
+
+    if (!options->flat_node_file.empty()) {
+        m_persistent_cache = std::make_shared<node_persistent_cache>(
+            options->flat_node_file, options->droptemp);
+    }
 }
 
 void middle_ram_t::set_requirements(output_requirements const &requirements)
@@ -53,6 +101,7 @@ void middle_ram_t::set_requirements(output_requirements const &requirements)
 
     log_debug("Middle 'ram' options:");
     log_debug("  locations: {}", m_store_options.locations);
+    log_debug("  locations_on_disk: {}", !!m_persistent_cache);
     log_debug("  way_nodes: {}", m_store_options.way_nodes);
     log_debug("  nodes: {}", m_store_options.nodes);
     log_debug("  untagged_nodes: {}", m_store_options.untagged_nodes);
@@ -66,8 +115,15 @@ void middle_ram_t::stop()
 
     auto const mbyte = 1024 * 1024;
 
-    log_debug("Middle 'ram': Node locations: size={} bytes={}M",
-              m_node_locations.size(), m_node_locations.used_memory() / mbyte);
+    if (m_persistent_cache) {
+        log_debug("Middle 'ram': Node locations on disk: size={} bytes={}M",
+                  m_persistent_cache->size(),
+                  m_persistent_cache->used_memory() / mbyte);
+    } else {
+        log_debug("Middle 'ram': Node locations in memory: size={} bytes={}M",
+                  m_node_locations.size(),
+                  m_node_locations.used_memory() / mbyte);
+    }
 
     log_debug("Middle 'ram': Way nodes data: size={} capacity={} bytes={}M",
               m_way_nodes_data.size(), m_way_nodes_data.capacity(),
@@ -133,29 +189,17 @@ bool middle_ram_t::get_object(osmium::item_type type, osmid_t id,
     return true;
 }
 
-static void add_delta_encoded_way_node_list(std::string *data,
-                                            osmium::WayNodeList const &wnl)
-{
-    assert(data);
-
-    // Add number of nodes in list
-    protozero::add_varint_to_buffer(data, wnl.size());
-
-    // Add delta encoded node ids
-    osmium::DeltaEncode<osmid_t> delta;
-    for (auto const &nr : wnl) {
-        protozero::add_varint_to_buffer(
-            data, protozero::encode_zigzag64(delta.update(nr.ref())));
-    }
-}
-
 void middle_ram_t::node(osmium::Node const &node)
 {
     assert(m_middle_state == middle_state::node);
     assert(node.visible());
 
     if (m_store_options.locations) {
-        m_node_locations.set(node.id(), node.location());
+        if (m_persistent_cache) {
+            m_persistent_cache->set(node.id(), node.location());
+        } else {
+            m_node_locations.set(node.id(), node.location());
+        }
     }
 
     if (m_store_options.nodes &&
@@ -190,6 +234,18 @@ void middle_ram_t::relation(osmium::Relation const &relation)
     }
 }
 
+void middle_ram_t::after_nodes()
+{
+    assert(m_middle_state == middle_state::node);
+#ifndef NDEBUG
+    m_middle_state = middle_state::way;
+#endif
+
+    if (!m_persistent_cache) {
+        m_node_locations.log_stats();
+    }
+}
+
 osmium::Location middle_ram_t::get_node_location(osmid_t id) const
 {
     return m_node_locations.get(id);
@@ -202,10 +258,19 @@ std::size_t middle_ram_t::nodes_get_list(osmium::WayNodeList *nodes) const
     std::size_t count = 0;
 
     if (m_store_options.locations) {
-        for (auto &nr : *nodes) {
-            nr.set_location(m_node_locations.get(nr.ref()));
-            if (nr.location().valid()) {
-                ++count;
+        if (m_persistent_cache) {
+            for (auto &nr : *nodes) {
+                nr.set_location(m_persistent_cache->get(nr.ref()));
+                if (nr.location().valid()) {
+                    ++count;
+                }
+            }
+        } else {
+            for (auto &nr : *nodes) {
+                nr.set_location(m_node_locations.get(nr.ref()));
+                if (nr.location().valid()) {
+                    ++count;
+                }
             }
         }
     }
@@ -213,6 +278,16 @@ std::size_t middle_ram_t::nodes_get_list(osmium::WayNodeList *nodes) const
     return count;
 }
 
+bool middle_ram_t::node_get(osmid_t id, osmium::memory::Buffer *buffer) const
+{
+    assert(buffer);
+
+    if (m_store_options.nodes) {
+        return get_object(osmium::item_type::node, id, buffer);
+    }
+    return false;
+}
+
 bool middle_ram_t::way_get(osmid_t id, osmium::memory::Buffer *buffer) const
 {
     assert(buffer);
@@ -223,27 +298,6 @@ bool middle_ram_t::way_get(osmid_t id, osmium::memory::Buffer *buffer) const
     return false;
 }
 
-static void
-get_delta_encoded_way_nodes_list(std::string const &data, std::size_t offset,
-                                 osmium::builder::WayBuilder *builder)
-{
-    assert(builder);
-
-    char const *begin = data.data() + offset;
-    char const *const end = data.data() + data.size();
-
-    auto count = protozero::decode_varint(&begin, end);
-
-    osmium::DeltaDecode<osmid_t> delta;
-    osmium::builder::WayNodeListBuilder wnl_builder{*builder};
-    while (count > 0) {
-        auto const val =
-            protozero::decode_zigzag64(protozero::decode_varint(&begin, end));
-        wnl_builder.add_node_ref(delta.update(val));
-        --count;
-    }
-}
-
 std::size_t
 middle_ram_t::rel_members_get(osmium::Relation const &rel,
                               osmium::memory::Buffer *buffer,
