@@ -19,11 +19,13 @@
 #include <osmium/memory/buffer.hpp>
 #include <osmium/osm.hpp>
 
+#include <cassert>
 #include <cstddef>
 #include <memory>
 #include <string>
 #include <utility>
 
+class node_persistent_cache;
 class thread_pool_t;
 
 /**
@@ -60,10 +62,14 @@ public:
     void way(osmium::Way const &way) override;
     void relation(osmium::Relation const &) override;
 
+    void after_nodes() override;
+
     osmium::Location get_node_location(osmid_t id) const override;
 
     std::size_t nodes_get_list(osmium::WayNodeList *nodes) const override;
 
+    bool node_get(osmid_t id, osmium::memory::Buffer *buffer) const override;
+
     bool way_get(osmid_t id, osmium::memory::Buffer *buffer) const override;
 
     size_t rel_members_get(osmium::Relation const &rel,
@@ -122,6 +128,10 @@ private:
 
     /// Options for this middle.
     middle_ram_options m_store_options;
+
+    /// File cache
+    std::shared_ptr<node_persistent_cache> m_persistent_cache;
+
 }; // class middle_ram_t
 
 #endif // OSM2PGSQL_MIDDLE_RAM_HPP
