@@ -16,9 +16,9 @@
 #include "flex-table-column.hpp"
 #include "flex-table.hpp"
 #include "geom.hpp"
+#include "idlist.hpp"
 #include "output.hpp"
 
-#include <osmium/index/id_set.hpp>
 #include <osmium/osm/item_type.hpp>
 
 #include <lua.hpp>
@@ -35,8 +35,6 @@ class geom_transform_t;
 class thread_pool_t;
 struct options_t;
 
-using idset_t = osmium::index::IdSetSmall<osmid_t>;
-
 /**
  * When C++ code is called from the Lua code we sometimes need to know
  * in what context this happens. These are the possible contexts.
@@ -100,7 +98,7 @@ public:
     /// Constructor for new objects
     output_flex_t(std::shared_ptr<middle_query_t> const &mid,
                   std::shared_ptr<thread_pool_t> thread_pool,
-                  options_t const &options);
+                  options_t const &options, properties_t const &properties);
 
     /// Constructor for cloned objects
     output_flex_t(output_flex_t const *other,
@@ -125,10 +123,13 @@ public:
 
     void after_nodes() override;
     void after_ways() override;
+    void after_relations() override;
 
     void wait() override;
 
-    idset_t const &get_marked_way_ids() override;
+    idlist_t const &get_marked_node_ids() override;
+    idlist_t const &get_marked_way_ids() override;
+
     void reprocess_marked() override;
 
     void pending_way(osmid_t id) override;
@@ -162,10 +163,8 @@ public:
     int app_define_table();
     int app_define_expire_output();
     int app_get_bbox();
-    int app_mark_way();
 
     int table_tostring();
-    int table_add_row();
     int table_insert();
     int table_name();
     int table_schema();
@@ -183,6 +182,9 @@ public:
 private:
     void select_relation_members();
 
+    /// Call a Lua function that was "prepared" earlier.
+    void call_lua_function(prepared_lua_function_t func);
+
     /**
      * Call a Lua function that was "prepared" earlier with the OSMObject
      * as its only parameter.
@@ -190,11 +192,13 @@ private:
     void call_lua_function(prepared_lua_function_t func,
                            osmium::OSMObject const &object);
 
-    /// Aquire the lua_mutex and the call `call_lua_function()`.
+    /// Aquire the lua_mutex and then call `call_lua_function()`.
+    void get_mutex_and_call_lua_function(prepared_lua_function_t func);
+
     void get_mutex_and_call_lua_function(prepared_lua_function_t func,
                                          osmium::OSMObject const &object);
 
-    void init_lua(std::string const &filename);
+    void init_lua(std::string const &filename, properties_t const &properties);
 
     // Get the flex table that is as first parameter on the Lua stack.
     flex_table_t const &get_table_from_param();
@@ -208,21 +212,6 @@ private:
     osmium::OSMObject const &
     check_and_get_context_object(flex_table_t const &table);
 
-    geom::geometry_t run_transform(reprojection const &proj,
-                                   geom_transform_t const *transform,
-                                   osmium::Node const &node);
-
-    geom::geometry_t run_transform(reprojection const &proj,
-                                   geom_transform_t const *transform,
-                                   osmium::Way const &way);
-
-    geom::geometry_t run_transform(reprojection const &proj,
-                                   geom_transform_t const *transform,
-                                   osmium::Relation const &relation);
-
-    template <typename OBJECT>
-    void add_row(table_connection_t *table_connection, OBJECT const &object);
-
     void delete_from_table(table_connection_t *table_connection,
                            pg_conn_t const &db_connection,
                            osmium::item_type type, osmid_t osm_id);
@@ -293,9 +282,10 @@ private:
     /// The connection to the database server.
     pg_conn_t m_db_connection;
 
-    // This is shared between all clones of the output and must only be
+    // These are shared between all clones of the output and must only be
     // accessed while protected using the lua_mutex.
-    std::shared_ptr<idset_t> m_stage2_way_ids = std::make_shared<idset_t>();
+    std::shared_ptr<idlist_t> m_stage2_node_ids = std::make_shared<idlist_t>();
+    std::shared_ptr<idlist_t> m_stage2_way_ids = std::make_shared<idlist_t>();
 
     std::shared_ptr<db_copy_thread_t> m_copy_thread;
 
@@ -309,20 +299,29 @@ private:
     relation_cache_t m_relation_cache;
     osmium::Node const *m_context_node = nullptr;
 
-    prepared_lua_function_t m_process_node{};
-    prepared_lua_function_t m_process_way{};
-    prepared_lua_function_t m_process_relation{};
-    prepared_lua_function_t m_select_relation_members{};
+    osmium::memory::Buffer m_area_buffer;
+
+    prepared_lua_function_t m_process_node;
+    prepared_lua_function_t m_process_way;
+    prepared_lua_function_t m_process_relation;
+
+    prepared_lua_function_t m_process_untagged_node;
+    prepared_lua_function_t m_process_untagged_way;
+    prepared_lua_function_t m_process_untagged_relation;
+
+    prepared_lua_function_t m_select_relation_members;
+
+    prepared_lua_function_t m_after_nodes;
+    prepared_lua_function_t m_after_ways;
+    prepared_lua_function_t m_after_relations;
 
     calling_context m_calling_context = calling_context::main;
 
     /**
      * This is set before calling stage1c process_relation() to disable the
-     * add_row() command.
+     * insert() command.
      */
-    bool m_disable_add_row = false;
-
-    bool m_add_row_has_never_been_called = true;
+    bool m_disable_insert = false;
 };
 
 #endif // OSM2PGSQL_OUTPUT_FLEX_HPP
