@@ -45,9 +45,10 @@
 #include "wildcmp.hpp"
 #include "wkb.hpp"
 
-static double calculate_area(bool reproject_area,
-                             geom::geometry_t const &geom4326,
-                             geom::geometry_t const &geom)
+namespace {
+
+double calculate_area(bool reproject_area, geom::geometry_t const &geom4326,
+                      geom::geometry_t const &projected_geom)
 {
     static thread_local auto const proj3857 =
         reprojection::create_projection(3857);
@@ -56,14 +57,45 @@ static double calculate_area(bool reproject_area,
         auto const ogeom = geom::transform(geom4326, *proj3857);
         return geom::area(ogeom);
     }
-    return geom::area(geom);
+    return geom::area(projected_geom);
 }
 
+// The roles of all available member ways of a relation are available in the
+// Lua "filter_tags_relation_member" callback function. This function extracts
+// the roles from all ways in the buffer and returns the list.
+rolelist_t get_rolelist(osmium::Relation const &rel,
+                        osmium::memory::Buffer const &buffer)
+{
+    rolelist_t roles;
+
+    auto it = buffer.select<osmium::Way>().cbegin();
+    auto const end = buffer.select<osmium::Way>().cend();
+
+    if (it == end) {
+        return roles;
+    }
+
+    for (auto const &member : rel.members()) {
+        if (member.type() == osmium::item_type::way &&
+            member.ref() == it->id()) {
+            roles.emplace_back(member.role());
+            ++it;
+            if (it == end) {
+                break;
+            }
+        }
+    }
+
+    return roles;
+}
+
+} // anonymous namespace
+
 void output_pgsql_t::pgsql_out_way(osmium::Way const &way, taglist_t *tags,
                                    bool polygon, bool roads)
 {
     if (polygon && !way.nodes().empty() && way.is_closed()) {
-        auto const geom = geom::create_polygon(way);
+        auto const geom = geom::create_polygon(way, &m_area_buffer);
         auto const projected_geom = geom::transform(geom, *m_proj);
 
         auto const wkb = geom_to_ewkb(projected_geom);
@@ -203,35 +235,6 @@ void output_pgsql_t::way_add(osmium::Way *way)
     }
 }
 
-// The roles of all available member ways of a relation are available in the
-// Lua "filter_tags_relation_member" callback function. This function extracts
-// the roles from all ways in the buffer and returns the list.
-static rolelist_t get_rolelist(osmium::Relation const &rel,
-                               osmium::memory::Buffer const &buffer)
-{
-    rolelist_t roles;
-
-    auto it = buffer.select<osmium::Way>().cbegin();
-    auto const end = buffer.select<osmium::Way>().cend();
-
-    if (it == end) {
-        return roles;
-    }
-
-    for (auto const &member : rel.members()) {
-        if (member.type() == osmium::item_type::way &&
-            member.ref() == it->id()) {
-            roles.emplace_back(member.role());
-            ++it;
-            if (it == end) {
-                break;
-            }
-        }
-    }
-
-    return roles;
-}
-
 /* This is the workhorse of pgsql_add_relation, split out because it is used as the callback for iterate relations */
 void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
 {
@@ -294,9 +297,9 @@ void output_pgsql_t::pgsql_process_relation(osmium::Relation const &rel)
 
     // multipolygons and boundaries
     if (make_boundary || make_polygon) {
-        auto const geoms =
-            geom::split_multi(geom::create_multipolygon(rel, m_buffer),
-                              !get_options()->enable_multi);
+        auto const geoms = geom::split_multi(
+            geom::create_multipolygon(rel, m_buffer, &m_area_buffer),
+            !get_options()->enable_multi);
         for (auto const &sgeom : geoms) {
             auto const projected_geom = geom::transform(sgeom, *m_proj);
             m_expire.from_geometry_if_3857(projected_geom, m_expire_config);
@@ -436,8 +439,13 @@ output_pgsql_t::output_pgsql_t(std::shared_ptr<middle_query_t> const &mid,
 : output_t(mid, std::move(thread_pool), options), m_proj(options.projection),
   m_expire(options.expire_tiles_zoom, options.projection),
   m_buffer(32768, osmium::memory::Buffer::auto_grow::yes),
-  m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
+  m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes),
+  m_area_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
 {
+    log_warn("The pgsql (default) output is deprecated. For details see "
+             "https://osm2pgsql.org/doc/"
+             "faq.html#the-pgsql-output-is-deprecated-what-does-that-mean");
+
     m_expire_config.full_area_limit = get_options()->expire_tiles_max_bbox;
     if (get_options()->expire_tiles_max_bbox > 0.0) {
         m_expire_config.mode = expire_mode::hybrid;
@@ -502,7 +510,8 @@ output_pgsql_t::output_pgsql_t(
   m_proj(get_options()->projection), m_expire_config(other->m_expire_config),
   m_expire(get_options()->expire_tiles_zoom, get_options()->projection),
   m_buffer(1024, osmium::memory::Buffer::auto_grow::yes),
-  m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
+  m_rels_buffer(1024, osmium::memory::Buffer::auto_grow::yes),
+  m_area_buffer(1024, osmium::memory::Buffer::auto_grow::yes)
 {
     for (std::size_t i = 0; i < m_tables.size(); ++i) {
         //copy constructor will just connect to the already there table
