@@ -12,17 +12,10 @@
 #include "db-copy.hpp"
 #include "format.hpp"
 #include "options.hpp"
-#include "output-gazetteer.hpp"
+#include "output-flex.hpp"
 #include "output-null.hpp"
 #include "output-pgsql.hpp"
 
-#ifdef HAVE_LUA
-# include "output-flex.hpp"
-static constexpr char const *const flex_backend = "flex, ";
-#else
-static constexpr char const *const flex_backend = "";
-#endif
-
 #include <stdexcept>
 #include <string>
 #include <utility>
@@ -30,23 +23,17 @@ static constexpr char const *const flex_backend = "";
 std::shared_ptr<output_t>
 output_t::create_output(std::shared_ptr<middle_query_t> const &mid,
                         std::shared_ptr<thread_pool_t> thread_pool,
-                        options_t const &options)
+                        options_t const &options,
+                        properties_t const &properties)
 {
     if (options.output_backend == "pgsql") {
         return std::make_shared<output_pgsql_t>(mid, std::move(thread_pool),
                                                 options);
     }
 
-#ifdef HAVE_LUA
     if (options.output_backend == "flex") {
         return std::make_shared<output_flex_t>(mid, std::move(thread_pool),
-                                               options);
-    }
-#endif
-
-    if (options.output_backend == "gazetteer") {
-        return std::make_shared<output_gazetteer_t>(mid, std::move(thread_pool),
-                                                    options);
+                                               options, properties);
     }
 
     if (options.output_backend == "null") {
@@ -55,8 +42,8 @@ output_t::create_output(std::shared_ptr<middle_query_t> const &mid,
     }
 
     throw fmt_error("Output backend '{}' not recognised. Should be one of"
-                    " [pgsql, {}gazetteer, null].",
-                    options.output_backend, flex_backend);
+                    " [pgsql, flex, null].",
+                    options.output_backend);
 }
 
 output_t::output_t(std::shared_ptr<middle_query_t> mid,
