@@ -7,15 +7,18 @@
  * For a full list of authors see the git log.
  */
 
+#include "pgsql-helper.hpp"
+
 #include "format.hpp"
 #include "pgsql.hpp"
-#include "pgsql-helper.hpp"
+#include "pgsql-capabilities.hpp"
 
 #include <cassert>
 
 idlist_t get_ids_from_result(pg_result_t const &result) {
-    idlist_t ids;
     assert(result.num_tuples() >= 0);
+
+    idlist_t ids;
     ids.reserve(static_cast<std::size_t>(result.num_tuples()));
 
     for (int i = 0; i < result.num_tuples(); ++i) {
@@ -72,3 +75,10 @@ void analyze_table(pg_conn_t const &db_connection, std::string const &schema,
     auto const qual_name = qualified_name(schema, name);
     db_connection.exec("ANALYZE {}", qual_name);
 }
+
+void drop_table_if_exists(pg_conn_t const &db_connection,
+                          std::string const &schema, std::string const &name)
+{
+    db_connection.exec("DROP TABLE IF EXISTS {} CASCADE",
+                       qualified_name(schema, name));
+}
