@@ -8,11 +8,9 @@
  */
 
 #include <algorithm>
-#include <cassert>
 #include <cstdint>
 #include <cstdio>
 #include <limits>
-#include <stdexcept>
 #include <string>
 #include <vector>
 
@@ -54,25 +52,26 @@ table_t::table_t(table_t const &other,
     // if the other table has already started, then we want to execute
     // the same stuff to get into the same state. but if it hasn't, then
     // this would be premature.
-    if (other.m_sql_conn) {
+    if (other.m_db_connection) {
         connect();
         prepare();
     }
 }
 
-void table_t::teardown() { m_sql_conn.reset(); }
+void table_t::teardown() { m_db_connection.reset(); }
 
 void table_t::sync() { m_copy.sync(); }
 
 void table_t::connect()
 {
-    m_sql_conn = std::make_unique<pg_conn_t>(m_connection_params, "out.pgsql");
+    m_db_connection =
+        std::make_unique<pg_conn_t>(m_connection_params, "out.pgsql");
 }
 
 void table_t::start(connection_params_t const &connection_params,
                     std::string const &table_space)
 {
-    if (m_sql_conn) {
+    if (m_db_connection) {
         throw fmt_error("{} cannot start, its already started.",
                         m_target->name());
     }
@@ -82,21 +81,22 @@ void table_t::start(connection_params_t const &connection_params,
 
     connect();
     log_info("Setting up table '{}'", m_target->name());
-    auto const qual_name = qualified_name(m_target->schema(), m_target->name());
-    auto const qual_tmp_name =
-        qualified_name(m_target->schema(), m_target->name() + "_tmp");
 
     // we are making a new table
     if (!m_append) {
-        m_sql_conn->exec("DROP TABLE IF EXISTS {} CASCADE", qual_name);
+        drop_table_if_exists(*m_db_connection, m_target->schema(),
+                             m_target->name());
     }
 
     // These _tmp tables can be left behind if we run out of disk space.
-    m_sql_conn->exec("DROP TABLE IF EXISTS {}", qual_tmp_name);
+    drop_table_if_exists(*m_db_connection, m_target->schema(),
+                         m_target->name() + "_tmp");
 
     //making a new table
     if (!m_append) {
         //define the new table
+        auto const qual_name =
+            qualified_name(m_target->schema(), m_target->name());
         auto sql =
             fmt::format("CREATE UNLOGGED TABLE {} (osm_id int8,", qual_name);
 
@@ -128,10 +128,10 @@ void table_t::start(connection_params_t const &connection_params,
         sql += m_table_space;
 
         //create the table
-        m_sql_conn->exec(sql);
+        m_db_connection->exec(sql);
 
         if (m_srid != "4326") {
-            create_geom_check_trigger(*m_sql_conn, m_target->schema(),
+            create_geom_check_trigger(*m_db_connection, m_target->schema(),
                                       m_target->name(), "ST_IsValid(NEW.way)");
         }
     }
@@ -143,9 +143,9 @@ void table_t::prepare()
 {
     //let postgres cache this query as it will presumably happen a lot
     auto const qual_name = qualified_name(m_target->schema(), m_target->name());
-    m_sql_conn->exec("PREPARE get_wkb(int8) AS"
-                     " SELECT way FROM {} WHERE osm_id = $1",
-                     qual_name);
+    m_db_connection->exec("PREPARE get_wkb(int8) AS"
+                          " SELECT way FROM {} WHERE osm_id = $1",
+                          qual_name);
 }
 
 void table_t::generate_copy_column_list()
@@ -187,56 +187,39 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
 
     if (!m_append) {
         if (m_srid != "4326") {
-            drop_geom_check_trigger(*m_sql_conn, m_target->schema(),
+            drop_geom_check_trigger(*m_db_connection, m_target->schema(),
                                     m_target->name());
         }
 
         log_info("Clustering table '{}' by geometry...", m_target->name());
 
-        std::string sql = fmt::format("CREATE TABLE {} {} AS SELECT * FROM {}",
-                                      qual_tmp_name, m_table_space, qual_name);
+        std::string const sql =
+            fmt::format("CREATE TABLE {} {} AS SELECT * FROM {} ORDER BY way",
+                        qual_tmp_name, m_table_space, qual_name);
 
-        auto const postgis_version = get_postgis_version();
+        m_db_connection->exec(sql);
 
-        sql += " ORDER BY ";
-        if (postgis_version.major == 2 && postgis_version.minor < 4) {
-            log_debug("Using GeoHash for clustering table '{}'",
-                      m_target->name());
-            if (m_srid == "4326") {
-                sql += "ST_GeoHash(way,10)";
-            } else {
-                sql += "ST_GeoHash(ST_Transform(ST_Envelope(way),4326),10)";
-            }
-            sql += " COLLATE \"C\"";
-        } else {
-            log_debug("Using native order for clustering table '{}'",
-                      m_target->name());
-            // Since Postgis 2.4 the order function for geometries gives
-            // useful results.
-            sql += "way";
-        }
-
-        m_sql_conn->exec(sql);
-
-        m_sql_conn->exec("DROP TABLE {}", qual_name);
-        m_sql_conn->exec(R"(ALTER TABLE {} RENAME TO "{}")", qual_tmp_name,
-                         m_target->name());
+        m_db_connection->exec("DROP TABLE {}", qual_name);
+        m_db_connection->exec(R"(ALTER TABLE {} RENAME TO "{}")", qual_tmp_name,
+                              m_target->name());
 
         log_info("Creating geometry index on table '{}'...", m_target->name());
 
         // Use fillfactor 100 for un-updatable imports
-        m_sql_conn->exec("CREATE INDEX ON {} USING GIST (way) {} {}", qual_name,
-                         (updateable ? "" : "WITH (fillfactor = 100)"),
-                         tablespace_clause(table_space_index));
+        m_db_connection->exec("CREATE INDEX ON {} USING GIST (way) {} {}",
+                              qual_name,
+                              (updateable ? "" : "WITH (fillfactor = 100)"),
+                              tablespace_clause(table_space_index));
 
         /* slim mode needs this to be able to apply diffs */
         if (updateable) {
             log_info("Creating osm_id index on table '{}'...",
                      m_target->name());
-            m_sql_conn->exec("CREATE INDEX ON {} USING BTREE (osm_id) {}",
-                             qual_name, tablespace_clause(table_space_index));
+            m_db_connection->exec("CREATE INDEX ON {} USING BTREE (osm_id) {}",
+                                  qual_name,
+                                  tablespace_clause(table_space_index));
             if (m_srid != "4326") {
-                create_geom_check_trigger(*m_sql_conn, m_target->schema(),
+                create_geom_check_trigger(*m_db_connection, m_target->schema(),
                                           m_target->name(),
                                           "ST_IsValid(NEW.way)");
             }
@@ -247,18 +230,18 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
             log_info("Creating hstore indexes on table '{}'...",
                      m_target->name());
             if (m_hstore_mode != hstore_column::none) {
-                m_sql_conn->exec("CREATE INDEX ON {} USING GIN (tags) {}",
-                                 qual_name,
-                                 tablespace_clause(table_space_index));
+                m_db_connection->exec("CREATE INDEX ON {} USING GIN (tags) {}",
+                                      qual_name,
+                                      tablespace_clause(table_space_index));
             }
             for (auto const &hcolumn : m_hstore_columns) {
-                m_sql_conn->exec(R"(CREATE INDEX ON {} USING GIN ("{}") {})",
-                                 qual_name, hcolumn,
-                                 tablespace_clause(table_space_index));
+                m_db_connection->exec(
+                    R"(CREATE INDEX ON {} USING GIN ("{}") {})", qual_name,
+                    hcolumn, tablespace_clause(table_space_index));
             }
         }
         log_info("Analyzing table '{}'...", m_target->name());
-        analyze_table(*m_sql_conn, m_target->schema(), m_target->name());
+        analyze_table(*m_db_connection, m_target->schema(), m_target->name());
     }
     teardown();
 }
@@ -444,6 +427,6 @@ void table_t::escape_type(std::string const &value, ColumnType flags)
 
 pg_result_t table_t::get_wkb(osmid_t id)
 {
-    return m_sql_conn->exec_prepared_as_binary("get_wkb", id);
+    return m_db_connection->exec_prepared_as_binary("get_wkb", id);
 }
 
