@@ -60,7 +60,7 @@ public:
     std::chrono::microseconds runtime() const noexcept { return m_result; }
 
 private:
-    std::future<std::chrono::microseconds> m_future{};
+    std::future<std::chrono::microseconds> m_future;
     std::chrono::microseconds m_result{};
 }; // class task_result_t
 
@@ -95,6 +95,7 @@ public:
      *         of the task can be queried.
      */
     template <typename TFunction>
+    // NOLINTNEXTLINE(cppcoreguidelines-missing-std-forward) false positive
     std::future<std::chrono::microseconds> submit(TFunction &&func)
     {
         std::packaged_task<std::chrono::microseconds()> task{
