@@ -12,10 +12,9 @@
 
 #include "format.hpp"
 
+#include <filesystem>
 #include <string>
 
-#include <boost/filesystem.hpp>
-
 namespace testing::cleanup {
 
 /**
@@ -44,9 +43,8 @@ private:
             return;
         }
 
-        boost::system::error_code ec;
-        boost::filesystem::remove(m_filename, ec);
-        if (ec && warn) {
+        std::error_code ec;
+        if (!std::filesystem::remove(m_filename, ec) && warn) {
             fmt::print(stderr, "WARNING: Unable to remove \"{}\": {}\n",
                        m_filename, ec.message());
         }
