@@ -17,12 +17,12 @@
 #include <osmium/osm/types_from_string.hpp>
 #include <osmium/visitor.hpp>
 
-#include "dependency-manager.hpp"
 #include "input.hpp"
 #include "middle-pgsql.hpp"
 #include "middle-ram.hpp"
 #include "osmdata.hpp"
 #include "output.hpp"
+#include "properties.hpp"
 #include "taginfo-impl.hpp"
 
 #include "common-pg.hpp"
@@ -35,14 +35,11 @@
 namespace testing {
 
 inline void parse_file(options_t const &options,
-                       std::unique_ptr<dependency_manager_t> dependency_manager,
                        std::shared_ptr<middle_t> const &mid,
                        std::shared_ptr<output_t> const &output,
                        char const *filename = nullptr, bool do_stop = true)
 {
-    osmdata_t osmdata{std::move(dependency_manager), mid, output, options};
-
-    osmdata.start();
+    osmdata_t osmdata{mid, output, options};
 
     std::string filepath{TESTDATA_DIR};
     if (filename) {
@@ -134,26 +131,21 @@ public:
                     std::initializer_list<std::string> input_data,
                     std::string const &format = "opl")
     {
-        options.connection_params = m_db.connection_params();
+        options.connection_params.merge_with(m_db.connection_params());
+
+        properties_t const properties{options.connection_params,
+                                      options.middle_dbschema};
 
         auto thread_pool = std::make_shared<thread_pool_t>(1U);
         auto middle = create_middle(thread_pool, options);
         middle->start();
 
         auto output = output_t::create_output(middle->get_query_instance(),
-                                              thread_pool, options);
+                                              thread_pool, options, properties);
 
         middle->set_requirements(output->get_requirements());
 
-        auto dependency_manager =
-            options.with_forward_dependencies
-                ? std::make_unique<full_dependency_manager_t>(middle)
-                : std::make_unique<dependency_manager_t>();
-
-        osmdata_t osmdata{std::move(dependency_manager), middle, output,
-                          options};
-
-        osmdata.start();
+        osmdata_t osmdata{middle, output, options};
 
         std::vector<osmium::io::File> files;
         for (auto const &data : input_data) {
@@ -173,22 +165,21 @@ public:
 
     void run_file(options_t options, char const *file = nullptr)
     {
-        options.connection_params = m_db.connection_params();
+        options.connection_params.merge_with(m_db.connection_params());
+
+        properties_t const properties{options.connection_params,
+                                      options.middle_dbschema};
 
         auto thread_pool = std::make_shared<thread_pool_t>(1U);
         auto middle = std::make_shared<middle_ram_t>(thread_pool, &options);
         middle->start();
 
         auto output = output_t::create_output(middle->get_query_instance(),
-                                              thread_pool, options);
+                                              thread_pool, options, properties);
 
         middle->set_requirements(output->get_requirements());
 
-        auto dependency_manager =
-            std::make_unique<full_dependency_manager_t>(middle);
-
-        parse_file(options, std::move(dependency_manager), middle, output,
-                   file);
+        parse_file(options, middle, output, file);
 
         middle->wait();
         output->wait();
