@@ -12,9 +12,13 @@
 #include "common-import.hpp"
 #include "common-options.hpp"
 
-static testing::db::import_t db;
+namespace {
 
-static char const *const conf_file = "test_output_flex.lua";
+testing::db::import_t db;
+
+char const *const conf_file = "test_output_flex.lua";
+
+} // anonymous namespace
 
 struct options_slim_default
 {
