@@ -14,7 +14,11 @@
 
 #include <array>
 
-static testing::db::import_t db;
+namespace {
+
+testing::db::import_t db;
+
+} // anonymous namespace
 
 TEST_CASE("compute Z order")
 {
