@@ -12,7 +12,11 @@
 #include "common-import.hpp"
 #include "pgsql.hpp"
 
-static testing::db::import_t const db;
+namespace {
+
+testing::db::import_t db;
+
+} // anonymous namespace
 
 TEST_CASE("Tablespace clause with no tablespace")
 {
@@ -138,7 +142,7 @@ TEST_CASE("create table and insert something")
 
 TEST_CASE("empty result object should return fatal status")
 {
-    pg_result_t result;
+    pg_result_t const result;
     REQUIRE(result.status() == PGRES_FATAL_ERROR);
     REQUIRE_FALSE(result);
     REQUIRE(result.num_fields() == 0);
